'*****************************************************************************
'                                                                            *
'  AVImake.bas                                                               *
'                                                                            *
'  This program creates an AVI file from an FLI or FLC file.                 *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "AVImake.exe"

#INCLUDE "FGWin.inc"
#INCLUDE "Comdlg32.inc"
#RESOURCE "AVImake.pbr"

'------------------------------------------------------------------------------
' Menu identifiers much match those in AVImake.h
%IDM_Convert = 1
%IDM_Exit    = 2

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  szClassName            = "FGavimake"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = VARPTR(szClassName)
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Create AVI File From Flic File", _ ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hMenu AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG
GLOBAL cxBuffer AS LONG, cyBuffer AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      cxBuffer = 32
      cyBuffer = 32
      hVB = fg_vballoc(cxBuffer, cyBuffer)
      fg_vbopen hVB
      fg_vbcolors

      fg_setcolor -1
      fg_fillpage

      hMenu = GetMenu(hWnd)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_COMMAND
      SELECT CASE wParam

        CASE %IDM_Convert
          CALL Convert_Click

        CASE %IDM_Exit
          DestroyWindow hWnd
      END SELECT
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

'------------------------------------------------------------------------------

SUB Convert_Click()

  LOCAL FileSpec AS STRING
  LOCAL mbString AS STRING

  DIM FileHeader(1:128) AS BYTE
  DIM ContextFlic(1:16) AS BYTE
  DIM ContextAVI(1:24) AS BYTE
  DIM Bitmap() AS BYTE

  ' open the flic file to convert
  FileSpec = ""
  IF OpenFileDialog(GetActiveWindow(), "Open File", FileSpec, "", _
    "flic files (*.fli,*.flc)|*.FLI;*.FLC", "fli", %OFN_READONLY) = 0 THEN EXIT SUB

  ' make sure it really is a flic file, and if so, open it
  IF fg_flichead(FileSpec, FileHeader(1)) < 0 THEN
    mbString = FileSpec + CHR$(13) + "is not an FLI or FLC file."
    MSGBOX mbString, %MB_ICONSTOP OR %MB_OK, "Error"
    EXIT SUB
  END IF
  fg_flicsize FileHeader(1), cxBuffer, cyBuffer
  CALL SwitchBuffers
  fg_flicopen FileSpec, ContextFlic(1)

  ' display the first flic frame
  fg_flicplay ContextFlic(1), 1, %FG_NODELAY
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1

  ' create an empty AVI file with the same name as the flic file,
  ' but with an .avi extension
  FileSpec = LEFT$(FileSpec, INSTR(FileSpec, ".")-1) + ".avi"
  IF fg_avimake(FileSpec, ContextAVI(1), -1, cxBuffer, cyBuffer, 8, 10000, 30) < 0 THEN
    mbString = "Cannot create AVI file" + CHR$(13) + FileSpec
    MSGBOX mbString, %MB_ICONSTOP OR %MB_OK, "Error"
    EXIT SUB
  END IF

  ' create a 256-color bitmap whose size is equal to the flic resolution
  REDIM Bitmap(1:fg_imagesiz(cxBuffer, cyBuffer))

  ' create the AVI file frame by frame
  MOUSEPTR 11
  fg_move 0, cyBuffer-1
  DO
    fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
    fg_getimage Bitmap(1), cxBuffer, cyBuffer
    fg_aviframe ContextAVI(1), Bitmap(1)
  LOOP WHILE fg_flicplay(ContextFlic(1), 1, %FG_NODELAY) = 1
  MOUSEPTR 0

  ' close the flic and AVI files, and release the bitmap memory
  fg_flicdone ContextFlic(1)
  fg_avidone ContextAVI(1)
  ERASE Bitmap

END SUB

'*****************************************************************************
'                                                                            *
'  SwitchBuffers()                                                           *
'                                                                            *
'  Close the and release the active virtual buffer, then create and open a   *
'  new virtual buffer to hold the new image file.                            *
'                                                                            *
'*****************************************************************************

SUB SwitchBuffers()

  fg_vbclose
  fg_vbfree hVB
  hVB = fg_vballoc(cxBuffer, cyBuffer)
  fg_vbopen hVB
  fg_vbcolors

END SUB
