'*****************************************************************************
'                                                                            *
'  Bitmap.bas                                                                *
'                                                                            *
'  This program demonstrates the Fastgraph for Windows 256-color bitmap      *
'  display functions.                                                        *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "Bitmap.exe"

#INCLUDE "FGWin.inc"
#RESOURCE "Bitmap.pbr"

%vbWidth = 320
%vbHeight = 240

DECLARE FUNCTION AppPath() AS STRING

GLOBAL Bird() AS BYTE

'------------------------------------------------------------------------------
' Menu identifiers much match those in Bitmap.h
%IDM_Drwimage = 1
%IDM_Clpimage = 2
%IDM_Revimage = 3
%IDM_Flpimage = 4
%IDM_Putimage = 5
%IDM_Exit     = 6

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  DIM Bird(1:40*20)

  szClassName            = "FGbitmap"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = VARPTR(szClassName)
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "256-Color Bitmap Demo", _   ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      hVB = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB
      fg_vbcolors

      fg_setcolor 20
      fg_fillpage

      CALL LoadBird
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_COMMAND
      SELECT CASE wParam

        CASE %IDM_Drwimage
          CALL Drwimage_Click

        CASE %IDM_Clpimage
          CALL Clpimage_Click

        CASE %IDM_Revimage
          CALL Revimage_Click

        CASE %IDM_Flpimage
          CALL Flpimage_Click

        CASE %IDM_Putimage
          CALL Putimage_Click

        CASE %IDM_Exit
          DestroyWindow hWnd
      END SELECT
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

SUB Clpimage_Click()
  fg_fillpage
  fg_move -20, %vbHeight/2 + 10
  fg_clpimage Bird(1), 40, 20
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

SUB Drwimage_Click()
  fg_fillpage
  fg_move(%vbWidth/2 - 20, %vbHeight/2 + 10
  fg_drwimage Bird(1), 40, 20
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

SUB Flpimage_Click()
  fg_fillpage
  fg_move %vbWidth-20, %vbHeight/2 + 10
  fg_flpimage Bird(1), 40, 20
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

SUB Putimage_Click()
  fg_fillpage
  fg_move %vbWidth/2 - 20, %vbHeight/2 + 10
  fg_putimage Bird(1), 40, 20
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

SUB Revimage_Click()
  fg_fillpage
  fg_move %vbWidth/2 - 20, %vbHeight/2 + 10
  fg_revimage Bird(1), 40, 20
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

FUNCTION AppPath() AS STRING
  LOCAL ModulePath AS ASCIIZ * 256
  GetModuleFileName %NULL, ModulePath, SIZEOF(ModulePath)
  FUNCTION = LEFT$(ModulePath, INSTR(-1, ModulePath, "\")-1)
END FUNCTION

SUB LoadBird()
  LOCAL I AS LONG
  OPEN AppPath()+"\Bird256.dat" FOR INPUT ACCESS READ AS #1
  FOR I = 1 TO 40*20
    INPUT #1, Bird(I)
  NEXT I
  CLOSE #1
END SUB
