'*****************************************************************************
'                                                                            *
'  Blend.bas                                                                 *
'                                                                            *
'  This program illustrates some of the Fastgraph for Windows alpha blending *
'  functions.                                                                *
'                                                                            *
'  Press F1 to view the foreground image.                                    *
'  Press F2 to view the background image.                                    *
'  Press F3 to create and view a 50% blended image.                          *
'  Press F4 to create and view a variable blended image.                     *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "Blend.exe"

#INCLUDE "FGWin.inc"

%vbWidth = 640
%vbHeight = 480
%vbDepth = 16

DECLARE FUNCTION AppPath() AS STRING

' direct color bitmap containing the foreground image
GLOBAL Foreground() AS BYTE

' direct color bitmap containing the background image
GLOBAL Background() AS BYTE

' direct color bitmap containing the resulting blended image
GLOBAL Blended() AS BYTE

' 256-color bitmap containing variable opacity values
GLOBAL Opacity() AS BYTE

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  DIM Foreground(1 : %vbWidth * %vbHeight * (%vbDepth/8))
  DIM Background(1 : %vbWidth * %vbHeight * (%vbDepth/8))
  DIM Blended(1 : %vbWidth * %vbHeight * (%vbDepth/8))
  DIM Opacity(1 : %vbWidth * %vbHeight)

  szClassName            = "FGblend"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = %NULL
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Alpha Blending Demo", _     ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      fg_vbdepth %vbDepth
      hVB = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB
      fg_vbcolors

      ' get background image from the CAT.BMP file
      fg_showbmp AppPath()+"\CAT.BMP", 0
      fg_move 0, %vbHeight-1
      fg_getdcb Background(1), %vbWidth, %vbHeight

      ' get foreground image from the PORCH.BMP file
      fg_showbmp AppPath()+"\PORCH.BMP", 0
      fg_move 0, %vbHeight-1
      fg_getdcb Foreground(1), %vbWidth, %vbHeight

      ' calcluate variable opacity bitmap
      CALL MakeOpacityBitmap

      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_KEYDOWN
      SELECT CASE wParam

        ' display foreground image
        CASE %VK_F1
          fg_move 0, %vbHeight-1
          fg_putdcb Foreground(1), %vbWidth, %vbHeight
          fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
          SetWindowText hWnd, "Alpha Blending: Foreground Image"

        ' display background image
        CASE %VK_F2
          fg_move 0, %vbHeight-1
          fg_putdcb Background(1), %vbWidth, %vbHeight
          fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
          SetWindowText hWnd, "Alpha Blending: Background Image"

        ' display blended image with constant 50% foreground opacity
        CASE %VK_F3
          MOUSEPTR 11
          fg_opacity 128
          fg_blenddcb Foreground(1), Background(1), Blended(1), %vbWidth * %vbHeight
          fg_move 0, %vbHeight-1
          fg_putdcb Blended(1), %vbWidth, %vbHeight
          fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
          SetWindowText hWnd, "Alpha Blending: 50% Blended Image"
          MOUSEPTR 0

        ' display blended image with variable foreground opacity
        CASE %VK_F4
          MOUSEPTR 11
          fg_blendvar Foreground(1), Background(1), Opacity(1), Blended(1), %vbWidth * %vbHeight
          fg_move 0, %vbHeight-1
          fg_putdcb Blended(1), %vbWidth, %vbHeight
          fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
          SetWindowText hWnd, "Alpha Blending: Variable Blended Image"
          MOUSEPTR 0

        CASE %VK_ESCAPE, %VK_F12
          DestroyWindow hWnd

      END SELECT
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      fg_modeset 0, 0, 0, 0
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

FUNCTION AppPath() AS STRING
  LOCAL ModulePath AS ASCIIZ * 256
  GetModuleFileName %NULL, ModulePath, SIZEOF(ModulePath)
  FUNCTION = LEFT$(ModulePath, INSTR(-1, ModulePath, "\")-1)
END FUNCTION

'*****************************************************************************
'                                                                            *
'  MakeOpacityBitmap()                                                       *
'                                                                            *
'  Define a 256-color bitmap with varying opacity values. The foregound      *
'  opacities will be zero at the image center and will gradually increase    *
'  as we move farther from the center.                                       *
'                                                                            *
'*****************************************************************************

SUB MakeOpacityBitmap()

   LOCAL I AS LONG, X AS LONG, Y AS LONG
   LOCAL OpacityValue AS LONG
   LOCAL yTerm AS LONG

   I = 0

   FOR Y = 0 TO %vbHeight-1
      yTerm = ABS(Y - %vbHeight/2)
      FOR X = 0 TO %vbWidth-1
         OpacityValue = ABS(X - %vbWidth/2) + yTerm
         IF (OpacityValue > 255) THEN
            Opacity(I) = 255
         ELSE
            Opacity(I) = OpacityValue
         END IF
         INCR I
      NEXT X
   NEXT Y

END SUB
