'*****************************************************************************
'                                                                            *
'  CBdemo.bas                                                                *
'                                                                            *
'  This program demonstrates how to exchange information between the active  *
'  virtual buffer and the Windows clipboard.                                 *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "CBdemo.exe"

#INCLUDE "FGWin.inc"
#RESOURCE "CBdemo.pbr"

%vbWidth = 320
%vbHeight = 200

DECLARE FUNCTION AppPath() AS STRING

'------------------------------------------------------------------------------
' Menu identifiers much match those in CBdemo.h
%IDM_Cut   = 1
%IDM_Paste = 2
%IDM_Exit  = 3

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  szClassName            = "FGcbdemo"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = VARPTR(szClassName)
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Clipboard Demo", _          ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hMenu AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      hVB = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB
      fg_vbcolors

      fg_showpcx AppPath()+"\MOUSE.PCX", %FG_AT_XY

      hMenu = GetMenu(hWnd)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_COMMAND
      SELECT CASE wParam

        CASE %IDM_Cut
          CALL Cut_Click
          EnableMenuItem hMenu, %IDM_Paste, %MF_ENABLED
          DrawMenuBar hWnd

        CASE %IDM_Paste
          CALL Paste_Click

        CASE %IDM_Exit
          DestroyWindow hWnd
      END SELECT
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

SUB Cut_Click()
  fg_vb2clip 0, %vbWidth-1, 0, %vbHeight-1
  fg_erase
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

SUB Paste_Click()
  fg_clip2vb 0, %vbWidth-1, 0, %vbHeight-1, 0
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

FUNCTION AppPath() AS STRING
  LOCAL ModulePath AS ASCIIZ * 256
  GetModuleFileName %NULL, ModulePath, SIZEOF(ModulePath)
  FUNCTION = LEFT$(ModulePath, INSTR(-1, ModulePath, "\")-1)
END FUNCTION
