'*****************************************************************************
'                                                                            *
'  Display.bas                                                               *
'                                                                            *
'  This program resizes the desktop to 800x600 using the fg_modeset()        *
'  function.                                                                 *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "Display.exe"

#INCLUDE "FGWin.inc"

%vbWidth = 800
%vbHeight = 600

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  szClassName            = "FGdisplay"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = %NULL
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindowEx( _
    %WS_EX_TOPMOST, _            ' extended window style
    szClassName, _               ' window class name
    "Change Display Settings", _ ' window caption
    %WS_POPUP, _                 ' window style
    0, _                         ' initial x position
    0, _                         ' initial y position
    %vbWidth, _                  ' initial x size
    %vbHeight, _                 ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      IF fg_modetest(%vbWidth, %vbHeight, fg_colors()) <> 0 THEN
        MessageBox hWnd, "Cannot set 800x600 desktop", "Error", %MB_OK OR %MB_ICONSTOP
        DestroyWindow hWnd
        FUNCTION = 0
        EXIT FUNCTION
      END IF
      ShowWindow hWnd, %SW_SHOWNORMAL
      fg_modeset %vbWidth, %vbHeight, fg_colors(), 1

      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      hVB = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB
      fg_vbcolors

      fg_setcolor 19
      fg_fillpage
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_KEYDOWN
      SELECT CASE wParam
        CASE %VK_ESCAPE, %VK_F12
          DestroyWindow hWnd
      END SELECT
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbpaste 0, %vbWidth-1, 0, %vbHeight-1, 0, %vbHeight-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      fg_modeset 0, 0, 0, 0
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION
