/****************************************************************************\
*                                                                            *
*  Fade.c                                                                    *
*                                                                            *
*  This program shows how perform a palette fade.                            *
*                                                                            *
\****************************************************************************/

#include <fgwin.h>
#include <string.h>
#include "Fade.h"

#define vbWidth  320
#define vbHeight 200

LRESULT CALLBACK WindowProc(HWND,UINT,WPARAM,LPARAM);

void FadeInClick();
void FadeOutClick();

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdParam, int iCmdShow)
{
   static char szAppName[] = "FGfade";
   HWND        hWnd;
   MSG         msg;
   WNDCLASSEX  wndclass;

   wndclass.cbSize        = sizeof(wndclass);
   wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
   wndclass.lpfnWndProc   = WindowProc;
   wndclass.cbClsExtra    = 0;
   wndclass.cbWndExtra    = 0;
   wndclass.hInstance     = hInstance;
   wndclass.hIcon         = LoadIcon(NULL,IDI_APPLICATION);
   wndclass.hCursor       = LoadCursor(NULL,IDC_ARROW);
   wndclass.hbrBackground = NULL;
   wndclass.lpszMenuName  = szAppName;
   wndclass.lpszClassName = szAppName;
   wndclass.hIconSm       = LoadIcon(NULL,IDI_APPLICATION);
   RegisterClassEx(&wndclass);

   hWnd = CreateWindow(szAppName, // window class name
      "Palette Fades",         // window caption
      WS_OVERLAPPEDWINDOW,     // window style
      CW_USEDEFAULT,           // initial x position
      CW_USEDEFAULT,           // initial y position
      CW_USEDEFAULT,           // initial x size
      CW_USEDEFAULT,           // initial y size
      NULL,                    // parent window handle
      NULL,                    // window menu handle
      hInstance,               // program instance handle
      NULL);                   // creation parameters

   ShowWindow(hWnd,iCmdShow);
   UpdateWindow(hWnd);

   while (GetMessage(&msg,NULL,0,0))
   {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }
   return msg.wParam;
}

/****************************************************************************\
*                                                                            *
*  WindowProc()                                                              *
*                                                                            *
\****************************************************************************/

HDC      hDC;
HMENU    hMenu;
HPALETTE hPal;
int      hVB;
UINT     cxClient, cyClient;
BYTE     Original[236*3];
BYTE     Current[236*3];

LRESULT CALLBACK WindowProc(HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
   PAINTSTRUCT ps;

   switch (iMsg)
   {
      case WM_CREATE:
         hDC = GetDC(hWnd);
         fg_setdc(hDC);
         hPal = fg_defpal();
         fg_realize(hPal);

         fg_vbinit();
         hVB = fg_vballoc(vbWidth,vbHeight);
         fg_vbopen(hVB);
         fg_vbcolors();

         fg_showpcx("MOUSE.PCX",FG_AT_XY);
         fg_getdacs(10,236,Original);

         hMenu = GetMenu(hWnd);
         return 0;

      case WM_COMMAND:
         switch (wParam)
         {
            case IDM_FadeIn:
               FadeInClick();
               return 0;

            case IDM_FadeOut:
               FadeOutClick();
               return 0;

            case IDM_Exit:
               DestroyWindow(hWnd);
               return 0;
         }
         break;

      case WM_PAINT:
         BeginPaint(hWnd,&ps);
         fg_vbscale(0,vbWidth-1,0,vbHeight-1,0,cxClient-1,0,cyClient-1);
         EndPaint(hWnd,&ps);
         return 0;

      case WM_SETFOCUS:
         fg_realize(hPal);
         InvalidateRect(hWnd,NULL,TRUE);
         return 0;

      case WM_SIZE:
         cxClient = LOWORD(lParam);
         cyClient = HIWORD(lParam);
         return 0;

      case WM_DESTROY:
         fg_vbclose();
         fg_vbfree(hVB);
         fg_vbfin();
         DeleteObject(hPal);
         ReleaseDC(hWnd,hDC);
         PostQuitMessage(0);
         return 0;
   }
   return DefWindowProc(hWnd,iMsg,wParam,lParam);
}

/****************************************************************************\
*                                                                            *
*  FadeInClick()                                                             *
*                                                                            *
*  Fade an image back to its original colors.                                *
*                                                                            *
\****************************************************************************/

void FadeInClick()
{
   HCURSOR hCursor;
   register int i, Fading;

   hCursor = SetCursor(LoadCursor(NULL,IDC_WAIT));
   ShowCursor(TRUE);
   memset(Current,0,236*3);

   do
   {
      Fading = FALSE;
      for (i = 0; i < 236*3; i++)
      {
         if (Current[i] != Original[i])
         {
            Current[i]++;
            Fading = TRUE;
         }
      }
      fg_setdacs(10,236,Current);
      if (fg_colors() > 8)
         fg_vbscale(0,vbWidth-1,0,vbHeight-1,0,cxClient-1,0,cyClient-1);
   }
   while (Fading);

   EnableMenuItem(hMenu,IDM_FadeIn,MF_GRAYED);
   EnableMenuItem(hMenu,IDM_FadeOut,MF_ENABLED);
   DrawMenuBar(GetActiveWindow());
   ShowCursor(FALSE);
   SetCursor(hCursor);
}

/****************************************************************************\
*                                                                            *
*  FadeOutClick()                                                            *
*                                                                            *
*  Fade an image to black.                                                   *
*                                                                            *
\****************************************************************************/

void FadeOutClick()
{
   HCURSOR hCursor;
   register int i, Fading;

   hCursor = SetCursor(LoadCursor(NULL,IDC_WAIT));
   ShowCursor(TRUE);
   memcpy(Current,Original,236*3);

   do
   {
      Fading = FALSE;
      for (i = 0; i < 236*3; i++)
      {
         if (Current[i] != 0)
         {
            Current[i]--;
            Fading = TRUE;
         }
      }
      fg_setdacs(10,236,Current);
      if (fg_colors() > 8)
         fg_vbscale(0,vbWidth-1,0,vbHeight-1,0,cxClient-1,0,cyClient-1);
   }
   while (Fading);

   EnableMenuItem(hMenu,IDM_FadeOut,MF_GRAYED);
   EnableMenuItem(hMenu,IDM_FadeIn,MF_ENABLED);
   DrawMenuBar(GetActiveWindow());
   ShowCursor(FALSE);
   SetCursor(hCursor);
}
