/****************************************************************************\
*                                                                            *
*  Fade.cs                                                                   *
*                                                                            *
*  This program shows how perform a palette fade.                            *
*                                                                            *
\****************************************************************************/
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.InteropServices;

namespace Fade
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		// Fastgraph variables.
		Graphics g;
		IntPtr hDC;
		int hPal;
		int hVB;
		int cxClient, cyClient;

		// Virtual buffer dimensions.
		const int vbWidth = 320;
		const int vbHeight = 200;

		// Holds original and current RGB palette values for 236 user colors.
		byte [] Original = new byte [236*3];
		byte [] Current = new byte [236*3];

		// Component declarations.
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem menuFadeIn;
		private System.Windows.Forms.MenuItem menuFadeOut;
		private System.Windows.Forms.MenuItem menuExit;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// Add any constructor code after InitializeComponent call
			//
			g = CreateGraphics();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			g.Dispose();
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.menuFadeIn = new System.Windows.Forms.MenuItem();
			this.menuFadeOut = new System.Windows.Forms.MenuItem();
			this.menuExit = new System.Windows.Forms.MenuItem();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuFadeIn,
																					  this.menuFadeOut,
																					  this.menuExit});
			// 
			// menuFadeIn
			// 
			this.menuFadeIn.Enabled = false;
			this.menuFadeIn.Index = 0;
			this.menuFadeIn.Text = "Fade&In";
			this.menuFadeIn.Click += new System.EventHandler(this.menuFadeIn_Click);
			// 
			// menuFadeOut
			// 
			this.menuFadeOut.Index = 1;
			this.menuFadeOut.Text = "Fade&Out";
			this.menuFadeOut.Click += new System.EventHandler(this.menuFadeOut_Click);
			// 
			// menuExit
			// 
			this.menuExit.Index = 2;
			this.menuExit.Text = "E&xit";
			this.menuExit.Click += new System.EventHandler(this.menuExit_Click);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(446, 276);
			this.Menu = this.mainMenu1;
			this.Name = "Form1";
			this.Text = "Palette Fades";
			this.Resize += new System.EventHandler(this.Form1_Resize);
			this.Load += new System.EventHandler(this.Form1_Load);
			this.Closed += new System.EventHandler(this.Form1_Closed);
			this.Activated += new System.EventHandler(this.Form1_Activated);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.Form1_Paint);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		private void Form1_Activated(object sender, System.EventArgs e)
		{
			fg.realize(hPal);
			cxClient = ClientRectangle.Width;
			cyClient = ClientRectangle.Height;
			Refresh();
		}

		private void Form1_Load(object sender, System.EventArgs e)
		{
			hDC = g.GetHdc();
			fg.setdc(hDC);
			hPal = fg.defpal();
			fg.realize(hPal);

			fg.vbinit();
			hVB = fg.vballoc(vbWidth, vbHeight);
			fg.vbopen(hVB);
			fg.vbcolors();

			fg.showpcx("..\\MOUSE.PCX", fg.AT_XY);
			fg.getdacs(10, 236, ref Original[0]);
		}

		private void Form1_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			fg.vbscale(0, vbWidth-1, 0, vbHeight-1, 0, cxClient-1, 0, cyClient-1);
		}

		private void Form1_Resize(object sender, System.EventArgs e)
		{
			cxClient = ClientRectangle.Width;
			cyClient = ClientRectangle.Height;
			fg.vbscale(0, vbWidth-1, 0, vbHeight-1, 0, cxClient-1, 0, cyClient-1);
		}

		private void Form1_Closed(object sender, System.EventArgs e)
		{
			fg.vbclose();
			fg.vbfree(hVB);
			fg.vbfin();
			g.ReleaseHdc(hDC);
		}

		/****************************************************************************\
		*                                                                            *
		*  FadeIn_Click()                                                            *
		*                                                                            *
		*  Fade an image back to its original colors.                                *
		*                                                                            *
		\****************************************************************************/

		private void menuFadeIn_Click(object sender, System.EventArgs e)
		{
			int i;
			bool Fading;

			this.Cursor = Cursors.WaitCursor;
			Array.Clear(Current, 0, Current.Length);

			do
			{
				Fading = false;
				for (i = 0; i < 236*3; i++)
				{
					if (Current[i] != Original[i])
					{
						Current[i]++;
						Fading = true;
					}
				}
				fg.setdacs(10, 236, ref Current[0]);
				if (fg.colors() > 8)
				{
					fg.vbscale(0, vbWidth-1, 0, vbHeight-1, 0, cxClient-1, 0, cyClient-1);
				}
			}
			while (Fading);

			menuFadeIn.Enabled = false;
			menuFadeOut.Enabled = true;
			this.Cursor = Cursors.Default;
		}

		/****************************************************************************\
		*                                                                            *
		*  FadeOut_Click()                                                           *
		*                                                                            *
		*  Fade an image to black.                                                   *
		*                                                                            *
		\****************************************************************************/

		private void menuFadeOut_Click(object sender, System.EventArgs e)
		{
			int i;
			bool Fading;

			this.Cursor = Cursors.WaitCursor;
			Current = (byte [])Original.Clone();

			do
			{
				Fading = false;
				for (i = 0; i < 236*3; i++)
				{
					if (Current[i] != 0)
					{
						Current[i]--;
						Fading = true;
					}
				}
				fg.setdacs(10, 236, ref Current[0]);
				if (fg.colors() > 8)
				{
					fg.vbscale(0, vbWidth-1, 0, vbHeight-1, 0, cxClient-1, 0, cyClient-1);
				}
			}
			while (Fading);

			menuFadeOut.Enabled = false;
			menuFadeIn.Enabled = true;
			this.Cursor = Cursors.Default;
		}

		private void menuExit_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}
	}
}