'*****************************************************************************
'                                                                            *
'  Fishtank.bas                                                              *
'                                                                            *
'  This program shows how to perform simple animation using Fastgraph for    *
'  Windows. Several types of tropical fish swim back and forth against a     *
'  coral reef background. The background image and fish sprites are stored   *
'  in PCX files.                                                             *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "Fishtank.exe"

#INCLUDE "FGWin.inc"

%nFish = 11                  ' total number of fish sprites

DECLARE FUNCTION AppPath() AS STRING

GLOBAL FishX() AS LONG       ' location of fish (x)
GLOBAL FishY() AS LONG       ' location of fish (y)
GLOBAL FishWidth() AS LONG   ' size of fish: width
GLOBAL FishHeight() AS LONG  ' size of fish: height
GLOBAL FishOffset() AS LONG  ' bitmap offsets into Fishes() array
GLOBAL Fishes() AS BYTE      ' bitmaps for all 6 kinds of fish

' There are 11 fish total, and 6 different kinds of fish. These
' arrays keep track of what kind of fish each fish is, and how each
' fish moves:

GLOBAL Fish() AS LONG        ' which fish bitmap applies to this fish?
GLOBAL X() AS LONG           ' starting x coordinate
GLOBAL Y() AS LONG           ' starting y coordinate
GLOBAL xMin() AS LONG        ' how far left (off screen) the fish can go
GLOBAL xMax() AS LONG        ' how far right (off screen) the fish can go
GLOBAL xInc() AS LONG        ' how fast the fish goes left and right
GLOBAL Dir() AS LONG         ' starting direction for each fish
GLOBAL yMin() AS LONG        ' how far up this fish can go
GLOBAL yMax() AS LONG        ' how far down this fish can go
GLOBAL yInc() AS LONG        ' how fast the fish moves up or down
GLOBAL yCount() AS LONG      ' counter to compare to yturn
GLOBAL yTurn() AS LONG       ' how long fish can go in the vertical direction
                             ' before stopping or turning around
'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  DIM FishX(0:5)
  DIM FishY(0:5)
  DIM FishWidth(0:5)
  DIM FishHeight(0:5)
  DIM FishOffset(0:5)

  DIM Fishes(0 : 56*25 + 54*38 + 68*26 + 56*30 + 62*22 + 68*36 - 1)

  DIM Fish(1:%nFish-1)
  DIM X(1:%nFish-1)
  DIM Y(1:%nFish-1)
  DIM xMin(1:%nFish-1)
  DIM xMax(1:%nFish-1)
  DIM xInc(1:%nFish-1)
  DIM Dir(1:%nFish-1)
  DIM yMin(1:%nFish-1)
  DIM yMax(1:%nFish-1)
  DIM yInc(1:%nFish-1)
  DIM yCount(1:%nFish-1)
  DIM yTurn(1:%nFish-1)

  szClassName            = "FGfishtank"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = %NULL
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Fastgraph Fish Tank", _     ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  ' The message loop processes entries placed in the message queue.
  ' When no message is ready, call GoFish() to perform one frame of
  ' animation

  WHILE %TRUE
    IF PeekMessage(Msg, %NULL, 0, 0, %PM_REMOVE) THEN
      IF Msg.message = %WM_QUIT THEN
        EXIT LOOP
      ELSE
        TranslateMessage Msg
        DispatchMessage Msg
      END IF
    ELSE
      CALL GoFish
    END IF
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB1 AS LONG, hVB2 AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT
  LOCAL bpp AS LONG

  SELECT CASE iMsg

    CASE %WM_CREATE
      ' use the default logical palette
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      ' create two 320x200 virtual buffers
      fg_vbinit
      hVB1 = fg_vballoc(320, 200)
      hVB2 = fg_vballoc(320, 200)

      ' display the coral background in virtual buffer #2 (which
      ' will always contain a clean copy of the background image)
      fg_vbopen hVB2
      fg_vbcolors
      fg_showpcx AppPath()+"\CORAL.PCX", %FG_AT_XY OR %FG_KEEPCOLORS

      ' get the fish bitmaps
      CALL GetFish
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, fg_getmaxx(), 0, fg_getmaxy(), 0, cxClient-1, 0, cyClient-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB1
      fg_vbfree hVB2
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

FUNCTION AppPath() AS STRING
  LOCAL ModulePath AS ASCIIZ * 256
  GetModuleFileName %NULL, ModulePath, SIZEOF(ModulePath)
  FUNCTION = LEFT$(ModulePath, INSTR(-1, ModulePath, "\")-1)
END FUNCTION

'*****************************************************************************
'                                                                            *
'  GetFish()                                                                 *
'                                                                            *
'  Fill the fish bitmap arrays.                                              *
'                                                                            *
'*****************************************************************************

SUB GetFish()

  LOCAL I AS LONG, J AS LONG

  FishX(0) = 0: FishX(1) = 64: FishX(2) = 128
  FishX(3) = 200: FishX(4) = 0: FishX(5) = 80

  FishY(0) = 199: FishY(1) = 199: FishY(2) = 199
  FishY(3) = 199: FishY(4) = 150: FishY(5) = 150

  FishWidth(0) = 56: FishWidth(1) = 54: FishWidth(2) = 68
  FishWidth(3) = 56: FishWidth(4) = 62: FishWidth(5) = 68

  FishHeight(0) = 25: FishHeight(1) = 38: FishHeight(2) = 26
  FishHeight(3) = 30: FishHeight(4) = 22: FishHeight(5) = 36

  Fish(0) = 2: Fish(1) = 2: Fish(2) = 3: Fish(3) = 4
  Fish(4) = 4: Fish(5) = 1: Fish(6) = 1: Fish(7) = 6
  Fish(8) = 5: Fish(9) = 3: Fish(10) = 4

  X(0) = -100: X(1) = -150: X(2) = -450: X(3) = -140
  X(4) = -200: X(5) = 520: X(6) = 620: X(7) = -800
  X(8) = 800: X(9) = 800: X(10) = -300

  Y(0) = 40: Y(1) = 60: Y(2) = 150: Y(3) = 80
  Y(4) = 70: Y(5) = 190: Y(6) = 180: Y(7) = 100
  Y(8) = 30: Y(9) = 130: Y(10) = 92

  xMin(0) = -300: xMin(1) = -300: xMin(2) = -800: xMin(3) = -200
  xMin(4) = -200: xMin(5) = -200: xMin(6) = -300: xMin(7) = -900
  xMin(8) = -900: xMin(9) = -900: xMin(10) = -400

  xMax(0) = 600: xMax(1) = 600: xMax(2) = 1100: xMax(3) = 1000
  xMax(4) = 1000: xMax(5) = 750: xMax(6) = 800: xMax(7) = 1200
  xMax(8) = 1400: xMax(9) = 1200: xMax(10) = 900

  xInc(0) = 2: xInc(1) = 2: xInc(2) = 8: xInc(3) = 5
  xInc(4) = 5: xInc(5) = -3: xInc(6) = -3: xInc(7) = 7
  xInc(8) = -8: xInc(9) = -9: xInc(10) = 6

  Dir(0) = 0: Dir(1) = 0: Dir(2) = 0: Dir(3) = 0
  Dir(4) = 0: Dir(5) = 1: Dir(6) = 1: Dir(7) = 0
  Dir(8) = 1: Dir(9) = 1: Dir(10) = 0

  yMin(0) = 40: yMin(1) = 60: yMin(2) = 120: yMin(3) = 70
  yMin(4) = 60: yMin(5) = 160: yMin(6) = 160: yMin(7) = 80
  yMin(8) = 30: yMin(9) = 110: yMin(10) = 72

  yMax(0) = 80: yMax(1) = 100: yMax(2) = 170: yMax(3) = 110
  yMax(4) = 100: yMax(5) = 199: yMax(6) = 199: yMax(7) = 120
  yMax(8) = 70: yMax(9) = 150: yMax(10) = 122

  yTurn(0) = 50: yTurn(1) = 30: yTurn(2) = 10: yTurn(3) = 30
  yTurn(4) = 20: yTurn(5) = 10: yTurn(6) = 10: yTurn(7) = 10
  yTurn(8) = 30: yTurn(9) = 20: yTurn(10) = 10

  RESET yCount()
  RESET yInc()

  ' get the fish bitmaps from a PCX file
  fg_vbopen hVB1
  fg_vbcolors
  fg_showpcx AppPath()+"\FISH.PCX", %FG_AT_XY OR %FG_IGNOREPALETTE OR %FG_KEEPCOLORS

  J = 0
  FOR I = 0 TO 5
     fg_move FishX(I), FishY(I)
     fg_getimage Fishes(J), FishWidth(I), FishHeight(I)
     FishOffset(I) = J
     J = J + FishWidth(I) * FishHeight(I)
  NEXT I

  fg_erase

END SUB

'*****************************************************************************
'                                                                            *
'  GoFish()                                                                  *
'                                                                            *
'  Make the fish swim around.                                                *
'                                                                            *
'*****************************************************************************

SUB GoFish()

  LOCAL I AS LONG

  ' copy the background to the workspace
  fg_copypage hVB2, hVB1

  ' put all the fish in their new positions
  FOR I = 0 TO %nFish-1
    INCR yCount(I)
    IF yCount(I) > yTurn(I) THEN
      yCount(I) = 0
      yInc(I) = INT(RND()*3 - 1)
    END IF
    Y(I) = Y(I) + yInc(I)
    Y(I) = MIN(yMax(I), MAX(Y(I), yMin(I)))

    IF X(I) >= -72 AND X(I) < 320 THEN
      CALL PutFish(Fish(I), X(I), Y(I), Dir(I))
    END IF

    X(I) = X(I) + xInc(I)
    IF X(I) <= xMin(I) OR X(I) >= xMax(I) THEN
      xInc(I) = -xInc(I)
      Dir(I) = 1 - Dir(I)
    END IF
  NEXT I

  ' scale the workspace image to fill the client area
  fg_vbscale 0, 319, 0, 199, 0, cxClient-1, 0, cyClient-1

END SUB

'*****************************************************************************
'                                                                            *
'  PutFish()                                                                 *
'                                                                            *
'  Draw one of the six fish anywhere you want.                               *
'                                                                            *
'*****************************************************************************

SUB PutFish(FishNum AS LONG, X AS LONG, Y AS LONG, FishDir AS LONG)

  LOCAL I AS LONG

  ' move to position where the fish will appear
  fg_move X, Y

  ' draw a left- or right-facing fish, depending on FishDir
  I = FishOffset(FishNum)
  IF FishDir = 0 THEN
    fg_flpimage Fishes(I), FishWidth(FishNum), FishHeight(FishNum)
  ELSE
    fg_clpimage Fishes(I), FishWidth(FishNum), FishHeight(FishNum)
  END IF
END SUB
