'*****************************************************************************
'                                                                            *
'  Fontdemo.bas                                                              *
'                                                                            *
'  This program shows how use Windows stock fonts in Fastgraph for Windows,  *
'  and also how to create and use a Windows logical font. The logical font   *
'  is a 24pt Arial font.                                                     *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "Fontdemo.exe"

#INCLUDE "FGWin.inc"

%vbWidth = 320
%vbHeight = 240

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  szClassName            = "FGfontdemo"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = %NULL
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Fastgraph for Windows Font Demo", _ ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hFont AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT
  LOCAL lf AS LOGFONT

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      hVB = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB
      fg_vbcolors

      fg_setcolor 24
      fg_fillpage

      lf.lfHeight = 24                 ' character height in pixels
      lf.lfWidth = 0                   ' default character width
      lf.lfEscapement = 0              ' orientation of next character
      lf.lfOrientation = 0             ' orientation of first character
      lf.lfWeight = %FW_NORMAL         ' normal thickness
      lf.lfItalic = %FALSE             ' not italic characters
      lf.lfUnderline = %FALSE          ' not underlined characters
      lf.lfStrikeOut = %FALSE          ' not strikeout characters
      lf.lfCharSet = %ANSI_CHARSET     ' ANSI character set
      lf.lfOutPrecision = 0            ' default output precision
      lf.lfClipPrecision = 0           ' default clipping precision
      lf.lfQuality = %DEFAULT_QUALITY  ' default scaling quality
      lf.lfPitchAndFamily = 0          ' default pitch and font family
      lf.lfFaceName = "Arial"          ' Arial typeface
      hFont = CreateFontIndirect(lf)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
      call DrawStrings
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hFont
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

SUB DrawStrings()
  LOCAL X AS LONG

  fg_setcolor 19
  X = fg_xclient(20)

  fg_move X, fg_yclient(20)
  fg_fontload 10
  fg_print "OEM fixed font", 14

  fg_move X, fg_yclient(40)
  fg_fontload 11
  fg_print "ANSI fixed font", 15

  fg_move X, fg_yclient(60)
  fg_fontload 12
  fg_print "ANSI var font", 13

  fg_move X, fg_yclient(80)
  fg_fontload 13
  fg_print "system font", 11

  fg_move X, fg_yclient(100)
  fg_fontload 14
  fg_print "device default font", 19

  fg_move X, fg_yclient(120)
  fg_fontload 16
  fg_print "system fixed font", 17

  fg_move X, fg_yclient(160)
  fg_logfont hFont
  fg_print "Arial 24 font", 13
END SUB
  