'*****************************************************************************
'                                                                            *
'  FrameDD.bas                                                               *
'                                                                            *
'  This program shows how to set up a full screen DirectDraw application     *
'  for either blitting or flipping. The selection of blitting or flipping is *
'  controlled by the %BLIT and %FLIP symbols defined below.                  *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "FrameDD.exe"

#INCLUDE "FGWinD.inc"

%vbWidth = 640
%vbHeight = 480

' define either %BLIT or %FLIP, but not both, for blitting or flipping
%BLIT = %TRUE
'%FLIP = %TRUE

GLOBAL AppIsActive AS LONG

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  AppIsActive = %FALSE

  szClassName            = "FGframedd"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = %NULL
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindowEx( _
    %WS_EX_TOPMOST, _            ' extended window style
    szClassName, _               ' window class name
    "DirectDraw Frame Animation", _  ' window caption
    %WS_POPUP, _                 ' window style
    0, _                         ' initial x position
    0, _                         ' initial y position
    %vbWidth, _                  ' initial x size
    %vbHeight, _                 ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  ' The message loop processes entries placed in the message queue.
  ' When no message is ready and this is the active application, call
  ' Animate() to perform one frame of animation.

  WHILE %TRUE
    IF PeekMessage(Msg, %NULL, 0, 0, %PM_REMOVE) THEN
      IF Msg.message = %WM_QUIT THEN
        EXIT LOOP
      ELSE
        TranslateMessage Msg
        DispatchMessage Msg
      END IF
    ELSEIF AppIsActive THEN
      CALL Animate
    END IF
  WEND


  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  SELECT CASE iMsg

    CASE %WM_ACTIVATEAPP
      AppIsActive = wParam
      IF AppIsActive THEN fg_ddrestore
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_CREATE
      #IF %DEF(%BLIT)
      fg_ddsetup %vbWidth, %vbHeight, 8, %FG_DX_BLIT
      #ELSE
      fg_ddsetup %vbWidth, %vbHeight, 8, %FG_DX_FLIP
      #ENDIF
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal
      ShowWindow hWnd, %SW_SHOWNORMAL

      ' if blitting, create a virtual buffer the same size as the screen
      ' resolution; if flipping, use the primary surface's back buffer
      fg_vbinit
      #IF %DEF(%BLIT)
      hVB = fg_vballoc(%vbWidth, %vbHeight)
      #ELSE
      hVB = 0
      #ENDIF
      fg_vbopen hVB
      fg_vbcolors

      fg_mouseini
      fg_mousevis 0
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_KEYDOWN
      SELECT CASE wParam
        CASE %VK_ESCAPE, %VK_F12
          DestroyWindow hWnd
      END SELECT
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_mousevis 1
      fg_vbclose
      #IF %DEF(%BLIT)
      fg_vbfree hVB
      #ENDIF
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

'****************************************************************************\
'                                                                            *
'  Animate()                                                                 *
'                                                                            *
'  Construct the next frame of animation and display it with either blitting *
'  or flipping, as directed by the %BLIT and %FLIP symbols above.            *
'                                                                            *
'****************************************************************************/

SUB Animate()

  LOCAL nColor AS LONG

  ' fill drawing surface with the next color
  nColor = (fg_getcolor() + 1) AND &hFF
  fg_setcolor nColor
  fg_fillpage

  ' blit or flip surface to the screen
  #IF %DEF(%BLIT)
  fg_vbpaste 0, %vbWidth-1, 0, %vbHeight-1, 0, %vbHeight-1
  #ELSE
  fg_ddflip
  #ENDIF

END SUB
