'*****************************************************************************
'                                                                            *
'  Geometry.bas                                                              *
'                                                                            *
'  This program shows how to display 3D objects in object space and 3D world *
'  space.                                                                    *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "Geometry.exe"

#INCLUDE "FGWin.inc"

%vbWidth = 300
%vbHeight = 300

GLOBAL Colors() AS LONG

' six faces of a 2x2x2 cube, defined in object coordinates
GLOBAL CubeFaces() AS DOUBLE

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  DIM Colors(1:6)
  DIM CubeFaces(1:12,1:6)

  szClassName            = "FGgeometry"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = %NULL
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "3D Geometry", _             ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL hZB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      CALL InitGlobals

      ' create the device context and logical palette
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      ' create and open the virtual buffer
      fg_vbinit
      hVB = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB
      fg_vbcolors

      ' fill the virtual buffer with white pixels
      fg_setcolor -1
      fg_fillpage

      ' create and open the z-buffer
      hZB = fg_zballoc(%vbWidth, %vbHeight)
      fg_zbopen hZB

      ' define 3D viewport and render state
      fg_3Dviewport 0, %vbWidth-1, 0, %vbHeight-1, 1.0
      fg_3Drenderstate %FG_ZBUFFER

      ' make the client area equal to the virtual buffer size
      ShowWindow hWnd, %SW_SHOWNORMAL
      CALL SetWindowSize(%vbWidth, %vbHeight)

      ' draw the cubes and coordinate axes
      CALL DrawCubes
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

'*****************************************************************************
'                                                                            *
'  DrawCubes()                                                               *
'                                                                            *
'  Draws two cubes, one in 3D world space and the other in object space,     *
'  along with 3D coordinate axes.                                            *
'                                                                            *
'*****************************************************************************

SUB DrawCubes()

  LOCAL I AS LONG

  ' set the point of view (POV)
  fg_3Dmove 4.0, 4.0, -15.0

  ' position a cube at z=20.0 with no rotation
  fg_3Dmoveobject 0.0, 0.0, 20.0

  ' draw the 3D coordinate axes in world space
  fg_setcolor 0
  fg_3Dline 0.0, 0.0, 0.0, 10.0, 0.0, 0.0
  fg_3Dline 0.0, 0.0, 0.0, 0.0, 10.0, 0.0
  fg_3Dline 0.0, 0.0, 0.0, 0.0, 0.0, 500.0

  ' draw all six faces in both cubes
  FOR I = 1 TO 6
    fg_setcolor Colors(I)
    fg_3Dpolygon CubeFaces(1,I), 4
    fg_3Dpolygonobject CubeFaces(1,I), 4
  NEXT

END SUB

'*****************************************************************************
'                                                                            *
'  InitGlobals()                                                             *
'                                                                            *
'  Initialize global variables and arrays. Called from WM_CREATE handler.    *
'                                                                            *
'*****************************************************************************

SUB InitGlobals()

  Colors(1) = 19: Colors(2) = 20: Colors(3) = 21:
  Colors(4) = 22: Colors(5) = 23: Colors(6) = 24:

  ' top
  CubeFaces(1,1) = -1.0: CubeFaces(2,1) = 1.0: CubeFaces(3,1) = 1.0
  CubeFaces(4,1) = 1.0: CubeFaces(5,1) = 1.0: CubeFaces(6,1) = 1.0
  CubeFaces(7,1) = 1.0: CubeFaces(8,1) = 1.0: CubeFaces(9,1) = -1.0
  CubeFaces(10,1) = -1.0: CubeFaces(11,1) = 1.0: CubeFaces(12,1) = -1.0

  ' front
  CubeFaces(1,2) = -1.0: CubeFaces(2,2) = 1.0: CubeFaces(3,2) = -1.0
  CubeFaces(4,2) = 1.0: CubeFaces(5,2) = 1.0: CubeFaces(6,2) = -1.0
  CubeFaces(7,2) = 1.0: CubeFaces(8,2) = -1.0: CubeFaces(9,2) = -1.0
  CubeFaces(10,2) = -1.0: CubeFaces(11,2) = -1.0: CubeFaces(12,2) = -1.0

  ' left
  CubeFaces(1,3) = -1.0: CubeFaces(2,3) = 1.0: CubeFaces(3,3) = 1.0
  CubeFaces(4,3) = -1.0: CubeFaces(5,3) = 1.0: CubeFaces(6,3) = -1.0
  CubeFaces(7,3) = -1.0: CubeFaces(8,3) = -1.0: CubeFaces(9,3) = -1.0
  CubeFaces(10,3) = -1.0: CubeFaces(11,3) = -1.0: CubeFaces(12,3) = 1.0

  ' right
  CubeFaces(1,4) = 1.0: CubeFaces(2,4) = 1.0: CubeFaces(3,4) = -1.0
  CubeFaces(4,4) = 1.0: CubeFaces(5,4) = 1.0: CubeFaces(6,4) = 1.0
  CubeFaces(7,4) = 1.0: CubeFaces(8,4) = -1.0: CubeFaces(9,4) = 1.0
  CubeFaces(10,4) = 1.0: CubeFaces(11,4) = -1.0: CubeFaces(12,4) = -1.0

  ' bottom
  CubeFaces(1,5) = -1.0: CubeFaces(2,5) = -1.0: CubeFaces(3,5) = -1.0
  CubeFaces(4,5) = 1.0: CubeFaces(5,5) = -1.0: CubeFaces(6,5) = -1.0
  CubeFaces(7,5) = 1.0: CubeFaces(8,5) = -1.0: CubeFaces(9,5) = 1.0
  CubeFaces(10,5) = -1.0: CubeFaces(11,5) = -1.0: CubeFaces(12,5) = 1.0

  ' back
  CubeFaces(1,6) = 1.0: CubeFaces(2,6) = 1.0: CubeFaces(3,6) = 1.0
  CubeFaces(4,6) = -1.0: CubeFaces(5,6) = 1.0: CubeFaces(6,6) = 1.0
  CubeFaces(7,6) = -1.0: CubeFaces(8,6) = -1.0: CubeFaces(9,6) = 1.0
  CubeFaces(10,6) = 1.0: CubeFaces(11,6) = -1.0: CubeFaces(12,6) = 1.0

END SUB

'*****************************************************************************
'                                                                            *
'  SetWindowSize()                                                           *
'                                                                            *
'  Sets the window size so the client area has the specified dimensions.     *
'                                                                            *
'*****************************************************************************

SUB SetWindowSize(ClientWidth AS LONG, ClientHeight AS LONG)

  LOCAL ClientRect AS RECT
  LOCAL WindowRect AS RECT
  LOCAL WindowWidth AS LONG, WindowHeight AS LONG

  GetClientRect GetActiveWindow(), ClientRect
  GetWindowRect GetActiveWindow(), WindowRect

  WindowWidth = ClientWidth _
    + (WindowRect.nRight - WindowRect.nLeft) _
    - (ClientRect.nRight - ClientRect.nLeft)
  WindowHeight = ClientHeight _
    + (WindowRect.nBottom - WindowRect.nTop) _
    - (ClientRect.nBottom - ClientRect.nTop)
  SetWindowPos GetActiveWindow(), %NULL, 0, 0, _
     WindowWidth, WindowHeight, %SWP_NOMOVE OR %SWP_NOZORDER

END SUB
