'*****************************************************************************
'                                                                            *
'  Graphix.bas                                                               *
'                                                                            *
'  This program demonstrates some of the Fastgraph for Windows graphics      *
'  primitive functions.                                                      *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "Graphix.exe"

#INCLUDE "FGWin.inc"
#RESOURCE "Graphix.pbr"

%vbWidth = 640
%vbHeight = 480

'------------------------------------------------------------------------------
' Menu identifiers much match those in Graphix.h
%IDM_Points     = 1
%IDM_Lines      = 2
%IDM_Rectangles = 3
%IDM_Circles    = 4
%IDM_Ellipses   = 5
%IDM_Polygons   = 6
%IDM_Paint      = 7
%IDM_Exit       = 8

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  szClassName            = "FGgraphix"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = VARPTR(szClassName)
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Graphics Primitives", _     ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %vbWidth, _                  ' initial x size
    %vbHeight, _                 ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      hVB = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB
      fg_vbcolors

      fg_setcolor 25
      fg_fillpage
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_COMMAND
      SELECT CASE wParam

        CASE %IDM_Points
          CALL Points_Click

        CASE %IDM_Lines
          CALL Lines_Click

        CASE %IDM_Rectangles
          CALL Rectangles_Click

        CASE %IDM_Circles
          CALL Circles_Click

        CASE %IDM_Ellipses
          CALL Ellipses_Click

        CASE %IDM_Polygons
          CALL Polygons_Click

        CASE %IDM_Paint
          CALL Paint_Click

        CASE %IDM_Exit
          DestroyWindow hWnd
      END SELECT
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      CALL Blit
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

'*****************************************************************************
'                                                                            *
'  Circles_Click()                                                           *
'                                                                            *
'  Draw a series of concentric circles.                                      *
'                                                                            *
'*****************************************************************************

SUB Circles_Click()

  LOCAL I AS LONG, Radius AS LONG

  fg_setcolor 11
  fg_fillpage

  ' draw 25 concentric circles at the center of the virtual buffer
  fg_move %vbWidth/2, %vbHeight/2
  Radius = 4
  fg_setcolor 25
  FOR I = 1 TO 25
    fg_circle Radius
    Radius = Radius + 8
  NEXT I

  CALL Blit

END SUB

'*****************************************************************************
'                                                                            *
'  Ellipses_Click()                                                          *
'                                                                            *
'  Draw a series of concentric ellipses.                                     *
'                                                                            *
'*****************************************************************************

SUB Ellipses_Click()

  LOCAL I AS LONG
  LOCAL Horiz AS LONG, Vert AS LONG

  fg_setcolor 11
  fg_fillpage

  ' draw 80 concentric ellipses at the center of the virtual buffer
  fg_move %vbWidth/2, %vbHeight/2
  Horiz = 4
  Vert = 1
  fg_setcolor 25
  FOR I = 1 TO 80
     fg_ellipse Horiz, Vert
     Horiz = Horiz + 3
     Vert = Vert + 1
  NEXT I

  CALL Blit

END SUB

'*****************************************************************************
'                                                                            *
'  Lines_Click()                                                             *
'                                                                            *
'  Draw a pattern of solid lines.                                            *
'                                                                            *
'*****************************************************************************

SUB Lines_Click()

  LOCAL X AS LONG, Y AS LONG
  LOCAL I AS LONG, X1 AS LONG, X2 AS LONG, Y1 AS LONG
  DIM LineColor(7) AS LONG

  LineColor(0) = 12: LineColor(1) = 11: LineColor(2) = 19: LineColor(3) = 21
  LineColor(4) = 21: LineColor(5) = 19: LineColor(6) = 11: LineColor(7) = 12

  fg_setcolor 25
  fg_fillpage

  ' draw horizontal lines
  FOR Y = 0 TO %vbHeight-1 STEP 40
    FOR I = 0 TO 7
      fg_setcolor LineColor(I)
      Y1 = Y + 3 * I
      fg_move 0, Y1
      fg_draw %vbWidth-1, Y1
    NEXT I
  NEXT Y

  ' draw vertical lines
  FOR X = 0 TO %vbWidth-1 STEP 60
    FOR I = 0 TO 7
      fg_setcolor LineColor(I)
      X1 = X + 3 * I
      fg_move X1, 0
      fg_draw X1, %vbHeight-1
    NEXT I
  NEXT X

  ' draw red diagonal lines
  fg_setcolor 22
  FOR X1 = -640 TO 639 STEP 60
    X2 = X1 + %vbHeight
    fg_move X1, 0
    fg_draw X2, %vbHeight
  NEXT X1
  FOR X1 = 0 TO 1279 STEP 60
    X2 = X1 - %vbHeight
    fg_move X1, 0
    fg_draw X2, %vbHeight
  NEXT X1

  CALL Blit

END SUB

'*****************************************************************************
'                                                                            *
'  Paint_Click()                                                             *
'                                                                            *
'  Demonstrate region fill.                                                  *
'                                                                            *
'*****************************************************************************

SUB Paint_Click()

  LOCAL X1 AS LONG, X2 AS LONG, Y1 AS LONG, Y2 AS LONG

  fg_setcolor 25
  fg_fillpage

  ' draw a rectangle
  X1 = 40
  X2 = %vbWidth - 40
  Y1 = 20
  Y2 = %vbHeight - 20
  fg_setcolor 21
  fg_rect X1, X2, Y1, Y2

  ' outline the rectangle
  fg_setcolor 10
  fg_box X1, X2, Y1, Y2

  ' draw the circle
  X1 = %vbWidth / 2
  Y1 = %vbHeight / 2
  fg_move X1, Y1
  fg_circle 80

  ' draw cross bars in the circle
  fg_move X1, Y1-80
  fg_draw X1, Y1+80
  fg_move X1-80, Y1
  fg_draw X1+80, Y1

  ' paint each quarter of the circle
  fg_setcolor 11
  fg_paint X1-6, Y1-6
  fg_setcolor 12
  fg_paint X1+6, Y1+6
  fg_setcolor 13
  fg_paint X1+6, Y1-6
  fg_setcolor 14
  fg_paint X1-6, Y1+6

  ' paint the area outside the box
  fg_setcolor 24
  fg_paint 41, 21

  CALL Blit

END SUB

'*****************************************************************************
'                                                                            *
'  Points_Click()                                                            *
'                                                                            *
'  Draw a pattern of points.                                                 *
'                                                                            *
'*****************************************************************************

SUB Points_Click()

  LOCAL X AS LONG, Y AS LONG

  ' fill the virtual buffer with yellow pixels
  fg_setcolor 24
  fg_fillpage

  ' draw the patterns of points
  fg_setcolor 19
  FOR X = 7 TO %vbWidth-1 STEP 20
    FOR Y = 0 TO %vbHeight-1 STEP 8
      fg_point X, Y
    NEXT Y
  NEXT X

  fg_setcolor 22
  FOR X = 17 TO %vbWidth-1 STEP 20
    FOR Y = 4 TO %vbHeight-1 STEP 8
      fg_point X, Y
    NEXT Y
  NEXT X

  CALL Blit

END SUB

'*****************************************************************************
'                                                                            *
'  Polygons_Click()                                                          *
'                                                                            *
'  Draw a grid of filled polygons.                                           *
'                                                                            *
'*****************************************************************************

SUB Polygons_Click()

  LOCAL I AS LONG, J AS LONG
  DIM xyDarkBlue(7) AS LONG
  DIM xyLightBlue(7) AS LONG
  DIM xyGreen(7) AS LONG

  xyDarkBlue(0) = 0: xyDarkBlue(1) = 16
  xyDarkBlue(2) = 24: xyDarkBlue(3) = 0
  xyDarkBlue(4) = 24: xyDarkBlue(5) = 40
  xyDarkBlue(6) = 0: xyDarkBlue(7) = 56

  xyLightBlue(0) = 24: xyLightBlue(1) = 0
  xyLightBlue(2) = 72: xyLightBlue(3) = 0
  xyLightBlue(4) = 72: xyLightBlue(5) = 40
  xyLightBlue(6) = 24: xyLightBlue(7) = 40

  xyGreen(0) = 0: xyGreen(1) = 56: xyGreen(2) = 24: xyGreen(3) = 40
  xyGreen(4) = 72: xyGreen(5) = 40: xyGreen(6) = 48: xyGreen(7) = 56

  fg_setcolor 25
  fg_fillpage

  ' draw 225 filled polygons (15 rows of 15)
  FOR J = 0 TO 14
    FOR I = 0 TO 14
      fg_polyoff I*72-J*24, J*56-I*16
      fg_setcolor 11
      fg_polyfill xyDarkBlue(0), BYVAL %NULL, 4
      fg_setcolor 19
      fg_polyfill xyLightBlue(0), BYVAL %NULL, 4
      fg_setcolor 20
      fg_polyfill xyGreen(0), BYVAL %NULL, 4
    NEXT I
  NEXT J

  CALL Blit

END SUB

'*****************************************************************************
'                                                                            *
'  Rectangles_Click()                                                        *
'                                                                            *
'  Draw a grid of filled rectangles.                                         *
'                                                                            *
'*****************************************************************************

SUB Rectangles_Click()

  LOCAL I AS LONG, J AS LONG
  LOCAL nColor AS LONG
  LOCAL X1 AS LONG, X2 AS LONG, Y1 AS LONG, Y2 AS LONG
  LOCAL xInc AS LONG, yInc AS LONG

  X1 = 0
  xInc = %vbWidth / 10
  X2 = xInc - 1
  Y1 = 0
  yInc = %vbHeight / 10
  Y2 = yInc - 1
  nColor = 10

  ' draw 100 filled rectangles (10 rows of 10)
  FOR I = 1 TO 10
    FOR J = 1 TO 10
      fg_setcolor nColor
      fg_rect X1, X2, Y1, Y2
      INCR nColor
      IF nColor > 24 THEN nColor = 10
      X1 = X1 + xInc
      X2 = X2 + xInc
    NEXT J
    X1 = 0
    X2 = xInc - 1
    Y1 = Y1 + yInc
    Y2 = Y2 + yInc
  NEXT I

  CALL Blit

END SUB

'*****************************************************************************
'                                                                            *
'  Blit()                                                                    *
'                                                                            *
'  Use fg_vbpaste() or fg_vbscale() to display the virtual buffer contents   *
'  in the client area, depending on the size of the client window.           *
'                                                                            *
'*****************************************************************************

SUB Blit()

  IF cxClient > %vbWidth OR cyClient > %vbHeight THEN  ' window larger than 640x480
    fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
  ELSE
    fg_vbpaste 0, %vbWidth-1, 0, %vbHeight-1, 0, cyClient-1
  END IF

END SUB
