'*****************************************************************************
'                                                                            *
'  Image.bas                                                                 *
'                                                                            *
'  This program demonstrates the Fastgraph for Windows image file display    *
'  and creation functions.                                                   *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "Image.exe"

#INCLUDE "FGWin.inc"
#INCLUDE "Comdlg32.inc"
#RESOURCE "Image.pbr"

GLOBAL Context() AS BYTE
GLOBAL FileHeader() AS BYTE

'------------------------------------------------------------------------------
' Menu identifiers much match those in Image.h

%IDM_BMP_Open     = 101
%IDM_BMP_Make     = 102
%IDM_BMP_Details  = 103

%IDM_PCX_Open     = 201
%IDM_PCX_Make     = 202
%IDM_PCX_Details  = 203

%IDM_JPEG_Open    = 301
%IDM_JPEG_Details = 302

%IDM_Flic_Open    = 401
%IDM_Flic_Play    = 402
%IDM_Flic_Frame   = 403
%IDM_Flic_Reset   = 404
%IDM_Flic_Details = 405

%IDM_AVI_Open     = 501
%IDM_AVI_Play     = 502
%IDM_AVI_Frame    = 503
%IDM_AVI_Reset    = 504
%IDM_AVI_Details  = 505

%IDM_Exit         = 6

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  DIM Context(1:48)
  DIM FileHeader(1:128)

  szClassName            = "FGimage"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = VARPTR(szClassName)
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Image File Demo", _         ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hMenu AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG
GLOBAL cxBuffer AS LONG, cyBuffer AS LONG

GLOBAL nColors AS LONG
GLOBAL nFrames AS LONG

GLOBAL FileSpec AS STRING
GLOBAL mbString AS STRING

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      cxBuffer = 32
      cyBuffer = 32
      hVB = fg_vballoc(cxBuffer, cyBuffer)
      fg_vbopen hVB
      fg_vbcolors

      fg_setcolor -1
      fg_fillpage

      hMenu = GetMenu(hWnd)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_COMMAND
      SELECT CASE wParam

        CASE %IDM_BMP_Open
          CALL BMPOpen_Click

        CASE %IDM_BMP_Make
          CALL BMPMake_Click

        CASE %IDM_BMP_Details
          CALL BMPDetails_Click

        CASE %IDM_PCX_Open
          CALL PCXOpen_Click

        CASE %IDM_PCX_Make
          CALL PCXMake_Click

        CASE %IDM_PCX_Details
          CALL PCXDetails_Click

        CASE %IDM_JPEG_Open
          CALL JPEGOpen_Click

        CASE %IDM_JPEG_Details
          CALL JPEGDetails_Click

        CASE %IDM_Flic_Open
          CALL FlicOpen_Click

        CASE %IDM_Flic_Play
          CALL FlicPlay_Click

        CASE %IDM_Flic_Frame
          CALL FlicFrame_Click

        CASE %IDM_Flic_Reset
          CALL FlicReset_Click

        CASE %IDM_Flic_Details
          CALL FlicDetails_Click

        CASE %IDM_AVI_Open
          CALL AVIOpen_Click

        CASE %IDM_AVI_Play
          CALL AVIPlay_Click

        CASE %IDM_AVI_Frame
          CALL AVIFrame_Click

        CASE %IDM_AVI_Reset
          CALL AVIReset_Click

        CASE %IDM_AVI_Details
          CALL AVIDetails_Click

        CASE %IDM_Exit
          DestroyWindow hWnd
      END SELECT
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      CALL CloseContext
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

'*****************************************************************************
'                                                                            *
'  Event handlers for the items on the BMP menu.                             *
'                                                                            *
'*****************************************************************************

SUB BMPOpen_Click()

  FileSpec = ""
  IF OpenFileDialog(GetActiveWindow(), "Open File", FileSpec, "", _
    "BMP files (*.bmp)|*.BMP", "bmp", %OFN_READONLY) = 0 THEN EXIT SUB
  IF fg_bmphead(FileSpec, FileHeader(1)) < 0 THEN
    mbString = FileSpec + CHR$(13) + "is not a BMP file."
    MSGBOX mbString, %MB_ICONSTOP OR %MB_OK, "Error"
    EXIT SUB
  END IF

  MOUSEPTR 11
  nColors = fg_bmppal(FileSpec, BYVAL %NULL)
  fg_bmpsize FileHeader(1), cxBuffer, cyBuffer
  CALL SwitchBuffers
  fg_showbmp FileSpec, 0
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
  MOUSEPTR 0

  EnableMenuItem hMenu, %IDM_BMP_Make, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_BMP_Details, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_PCX_Make, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_PCX_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_JPEG_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Play, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Frame, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Reset, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Play, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Frame, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Reset, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Details, %MF_GRAYED

END SUB

'------------------------------------------------------------------------------

SUB BMPMake_Click()

  LOCAL ColorDepth AS LONG

  FileSpec = LEFT$(FileSpec, INSTR(FileSpec, ".")-1) + ".bmp"
  IF SaveFileDialog(GetActiveWindow(), "Save File", FileSpec, "", _
    "BMP files (*.bmp)|*.BMP", "bmp", _
    %OFN_HIDEREADONLY OR %OFN_OVERWRITEPROMPT OR %OFN_PATHMUSTEXIST) = 0 THEN EXIT SUB

  IF nColors = 0 THEN
    ColorDepth = 24
  ELSEIF nColors = 256 THEN
    ColorDepth = 8
  ELSEIF nColors = 16 THEN
    ColorDepth = 4
  ELSE
    ColorDepth = 1
  END IF
  MOUSEPTR 11
  fg_makebmp 0, cxBuffer-1, 0, cyBuffer-1, ColorDepth, FileSpec
  MOUSEPTR 0

END SUB

'------------------------------------------------------------------------------

SUB BMPDetails_Click()

  mbString = FileSpec + CHR$(13) + _
             STR$(cxBuffer) + " x" + STR$(cyBuffer) + " pixels" + CHR$(13)
  IF nColors > 0 THEN
    mbString = mbString + STR$(nColors) + " colors"
  ELSE
    mbString = mbString + "24-bit RGB"
  END IF
  MSGBOX mbString, %MB_ICONINFORMATION OR %MB_OK, "Information"

END SUB

'*****************************************************************************
'                                                                            *
'  Event handlers for the items on the PCX menu.                             *
'                                                                            *
'*****************************************************************************

SUB PCXOpen_Click()

  FileSpec = ""
  IF OpenFileDialog(GetActiveWindow(), "Open File", FileSpec, "", _
    "PCX files (*.pcx)|*.PCX", "pcx", %OFN_READONLY) = 0 THEN EXIT SUB
  IF fg_pcxhead(FileSpec, FileHeader(1)) < 0 THEN
    mbString = FileSpec + CHR$(13) + "is not a PCX file."
    MSGBOX mbString, %MB_ICONSTOP OR %MB_OK, "Error"
    EXIT SUB
  END IF

  MOUSEPTR 11
  nColors = fg_pcxpal(FileSpec, BYVAL %NULL)
  fg_pcxsize FileHeader(1), cxBuffer, cyBuffer
  CALL SwitchBuffers
  fg_showpcx FileSpec, %FG_AT_XY
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
  MOUSEPTR 0

  EnableMenuItem hMenu, %IDM_BMP_Make, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_BMP_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_PCX_Make, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_PCX_Details, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_JPEG_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Play, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Frame, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Reset, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Play, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Frame, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Reset, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Details, %MF_GRAYED

END SUB

'------------------------------------------------------------------------------

SUB PCXMake_Click()

  FileSpec = LEFT$(FileSpec, INSTR(FileSpec, ".")-1) + ".pcx"
  IF SaveFileDialog(GetActiveWindow(), "Save File", FileSpec, "", _
    "PCX files (*.pcx)|*.PCX", "pcx", _
    %OFN_HIDEREADONLY OR %OFN_OVERWRITEPROMPT OR %OFN_PATHMUSTEXIST) = 0 THEN EXIT SUB

  MOUSEPTR 11
  fg_makepcx 0, cxBuffer-1, 0, cyBuffer-1, FileSpec
  MOUSEPTR 0

END SUB

'------------------------------------------------------------------------------

SUB PCXDetails_Click()

  mbString = FileSpec + CHR$(13) + _
             STR$(cxBuffer) + " x" + STR$(cyBuffer) + " pixels" + CHR$(13)
  IF nColors > 0 THEN
    mbString = mbString + STR$(nColors) + " colors"
  ELSE
    mbString = mbString + "24-bit RGB"
  END IF
  MSGBOX mbString, %MB_ICONINFORMATION OR %MB_OK, "Information"

END SUB

'*****************************************************************************
'                                                                            *
'  Event handlers for the items on the JPEG menu.                            *
'                                                                            *
'*****************************************************************************

SUB JPEGOpen_Click()

  FileSpec = ""
  IF OpenFileDialog(GetActiveWindow(), "Open File", FileSpec, "", _
    "JPEG files (*.jpg)|*.JPG", "jpg", %OFN_READONLY) = 0 THEN EXIT SUB
  IF fg_jpeghead(FileSpec, FileHeader(1)) < 0 THEN
    mbString = FileSpec + CHR$(13) + "is not a baseline JPEG file."
    MSGBOX mbString, %MB_ICONSTOP OR %MB_OK, "Error"
    EXIT SUB
  END IF

  MOUSEPTR 11
  nColors = 0
  fg_jpegsize FileHeader(1), cxBuffer, cyBuffer
  CALL SwitchBuffers
  fg_showjpeg FileSpec, 0
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
  MOUSEPTR 0

  EnableMenuItem hMenu, %IDM_BMP_Make, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_BMP_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_PCX_Make, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_PCX_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_JPEG_Details, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Flic_Play, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Frame, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Reset, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Play, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Frame, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Reset, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Details, %MF_GRAYED

END SUB

'------------------------------------------------------------------------------

SUB JPEGDetails_Click()

  mbString = FileSpec + CHR$(13) + _
             STR$(cxBuffer) + " x" + STR$(cyBuffer) + " pixels" + CHR$(13) + _
             "24-bit RGB"
  MSGBOX mbString, %MB_ICONINFORMATION OR %MB_OK, "Information"

END SUB

'*****************************************************************************
'                                                                            *
'  Event handlers for the items on the FLI/FLC menu.                         *
'                                                                            *
'*****************************************************************************

SUB FlicOpen_Click()

  FileSpec = ""
  IF OpenFileDialog(GetActiveWindow(), "Open File", FileSpec, "", _
    "flic files (*.fli,*.flc)|*.FLI;*.FLC", "fli", %OFN_READONLY) = 0 THEN EXIT SUB
  IF fg_flichead(FileSpec, FileHeader(1)) < 0 THEN
    mbString = FileSpec + CHR$(13) + "is not an FLI or FLC file."
    MSGBOX mbString, %MB_ICONSTOP OR %MB_OK, "Error"
    EXIT SUB
  END IF

  nColors = 256
  fg_flicsize FileHeader(1), cxBuffer, cyBuffer
  CALL SwitchBuffers
  fg_flicopen FileSpec, Context(1)
  fg_flicplay Context(1), 1, %FG_NODELAY
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
  nFrames = FileHeader(8)*256 + FileHeader(7)

  EnableMenuItem hMenu, %IDM_BMP_Make, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_BMP_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_PCX_Make, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_PCX_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_JPEG_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Play, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Flic_Frame, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Flic_Reset, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Flic_Details, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_AVI_Play, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Frame, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Reset, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Details, %MF_GRAYED

END SUB

'------------------------------------------------------------------------------

SUB FlicPlay_Click()

  MOUSEPTR 11
  fg_showflic FileSpec, 0, %FG_NODELAY
  MOUSEPTR 0
  fg_flicskip Context(1), -1

END SUB

'------------------------------------------------------------------------------

SUB FlicFrame_Click()

  IF fg_flicplay(Context(1), 1, %FG_NODELAY) = 0 THEN
    fg_flicskip Context(1), -1
    fg_flicplay Context(1), 1, %FG_NODELAY
  END IF
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1

END SUB

'------------------------------------------------------------------------------

SUB FlicReset_Click()

  fg_flicskip Context(1), -1
  fg_flicplay Context(1), 1, %FG_NODELAY
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1

END SUB

'------------------------------------------------------------------------------

SUB FlicDetails_Click()

  mbString = FileSpec + CHR$(13) + _
             STR$(cxBuffer) + " x" + STR$(cyBuffer) + " pixels" + CHR$(13) + _
             STR$(nFrames) + " frames"
  MSGBOX mbString, %MB_ICONINFORMATION OR %MB_OK, "Information"

END SUB

'*****************************************************************************
'                                                                            *
'  Event handlers for the items on the AVI menu.                             *
'                                                                            *
'*****************************************************************************

SUB AVIOpen_Click()

  FileSpec = ""
  IF OpenFileDialog(GetActiveWindow(), "Open File", FileSpec, "", _
    "AVI files (*.avi)|*.AVI", "avi", %OFN_READONLY) = 0 THEN EXIT SUB
  IF fg_avihead(FileSpec, FileHeader(1)) < 0 THEN
    mbString = FileSpec + CHR$(13) + "is not an AVI file."
    MSGBOX mbString, %MB_ICONSTOP OR %MB_OK, "Error"
    EXIT SUB
  END IF

  nColors = fg_avipal(FileSpec, BYVAL %NULL)
  fg_avisize FileHeader(1), cxBuffer, cyBuffer
  CALL SwitchBuffers
  IF fg_aviopen(FileSpec, Context(1)) < 0 THEN
    mbString = "Cannot an AVI file " + FileSpec
    MSGBOX mbString, %MB_ICONSTOP OR %MB_OK, "Error"
    EnableMenuItem hMenu, %IDM_AVI_Play, %MF_GRAYED
    EnableMenuItem hMenu, %IDM_AVI_Frame, %MF_GRAYED
    EnableMenuItem hMenu, %IDM_AVI_Reset, %MF_GRAYED
    EnableMenuItem hMenu, %IDM_AVI_Details, %MF_GRAYED
    EXIT SUB
  END IF
  fg_aviplay Context(1), 1, %FG_NODELAY
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1

  EnableMenuItem hMenu, %IDM_BMP_Make, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_BMP_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_PCX_Make, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_PCX_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_JPEG_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Play, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Frame, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Reset, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Flic_Details, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_AVI_Play, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_AVI_Frame, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_AVI_Reset, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_AVI_Details, %MF_ENABLED

END SUB

'------------------------------------------------------------------------------

SUB AVIPlay_Click()

  MOUSEPTR 11
  fg_showavi FileSpec, 0, %FG_NODELAY
  MOUSEPTR 0
  fg_aviskip Context(1), -1

END SUB

'------------------------------------------------------------------------------

SUB AVIFrame_Click()

  IF fg_aviplay(Context(1), 1, %FG_NODELAY) = 0 THEN
    fg_aviskip Context(1), -1
    fg_aviplay Context(1), 1, %FG_NODELAY
  END IF
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1

END SUB

'------------------------------------------------------------------------------

SUB AVIReset_Click()

  fg_aviskip Context(1), -1
  fg_aviplay Context(1), 1, %FG_NODELAY
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1

END SUB

'------------------------------------------------------------------------------

SUB AVIDetails_Click()

  mbString = FileSpec + CHR$(13) + _
             STR$(cxBuffer) + " x" + STR$(cyBuffer) + " pixels" + CHR$(13)
  IF nColors > 0 THEN
    mbString = mbString + STR$(nColors) + " colors"
  ELSE
    mbString = mbString + "24-bit RGB"
  END IF
  MSGBOX mbString, %MB_ICONINFORMATION OR %MB_OK, "Information"

END SUB

'*****************************************************************************
'                                                                            *
'  CloseContext()                                                            *
'                                                                            *
'  Closes the active flic or AVI context. This function is called from       *
'  SwitchBuffers() and also from the WM_DESTROY handler.                     *
'                                                                            *
'*****************************************************************************

SUB CloseContext()

  IF (NOT(GetMenuState(hMenu, %IDM_Flic_Details, %MF_BYCOMMAND) AND 3)) <> 0 THEN
    fg_flicdone Context(1)
  ELSEIF (NOT(GetMenuState(hMenu, %IDM_AVI_Details, %MF_BYCOMMAND) AND 3)) <> 0 THEN
    fg_avidone Context(1)
  END IF

END SUB

'*****************************************************************************
'                                                                            *
'  SwitchBuffers()                                                           *
'                                                                            *
'  Close the and release the active virtual buffer, then create and open a   *
'  new virtual buffer to hold the new image file.                            *
'                                                                            *
'*****************************************************************************

SUB SwitchBuffers()

  CALL CloseContext
  fg_vbclose
  fg_vbfree hVB
  IF nColors = 0 THEN
    fg_vbdepth 24
  ELSE
    fg_vbdepth 8
  END IF
  hVB = fg_vballoc(cxBuffer, cyBuffer)
  fg_vbopen hVB
  fg_vbcolors

END SUB
