'*****************************************************************************
'                                                                            *
'  KBdemo.bas                                                                *
'                                                                            *
'  This program shows how to pan the contents of a virtual buffer through    *
'  a smaller window using the low-level keyboard handler.                    *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "KBdemo.exe"

#INCLUDE "FGWin.inc"

%vbWidth = 640
%vbHeight = 480

DECLARE FUNCTION AppPath() AS STRING

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  szClassName            = "FGkbdemo"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = %NULL
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Keyboard Handler Demo", _   ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %vbWidth/2, _                ' initial x size
    %vbHeight/2, _               ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  ' The message loop processes entries placed in the message queue.
  ' When no message is ready, call CheckForPanning() to check if we
  ' want to perform panning.

  WHILE %TRUE
    IF PeekMessage(Msg, %NULL, 0, 0, %PM_REMOVE) THEN
      IF Msg.message = %WM_QUIT THEN
        EXIT LOOP
      ELSE
        TranslateMessage Msg
        DispatchMessage Msg
      END IF
    ELSE
      CALL CheckForPanning
    END IF
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG

GLOBAL X AS LONG, Y AS LONG
GLOBAL xLimit AS LONG, yLimit AS LONG
GLOBAL CanGoLeft AS LONG, CanGoRight AS LONG
GLOBAL CanGoUp AS LONG, CanGoDown AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      hVB = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB
      fg_vbcolors

      fg_showbmp AppPath()+"\PORCH.BMP", 0
      x = 0
      y = 0
      CanGoRight = %TRUE
      CanGoDown  = %TRUE
      CanGoLeft  = %FALSE
      CanGoUp  = %FALSE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbpaste X, X+(%vbWidth-1), Y, Y+(%vbHeight-1), 0, %vbHeight-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      IF cxClient < %vbWidth THEN
        xLimit = %vbWidth - cxClient
        IF X > 0 THEN CanGoLeft = %TRUE
        IF X < xLimit THEN CanGoRight = %TRUE
      ELSE
        xLimit = 0
        CanGoLeft = %FALSE
        CanGoRight = %FALSE
      END IF
      IF cyClient < %vbHeight THEN
        yLimit = %vbHeight - cyClient
        IF Y > 0 THEN CanGoUp = %TRUE
        IF Y < yLimit THEN CanGoDown = %TRUE
      ELSE
        yLimit = 0
        CanGoUp = %FALSE
        CanGoDown = %FALSE
      END IF
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

FUNCTION AppPath() AS STRING
  LOCAL ModulePath AS ASCIIZ * 256
  GetModuleFileName %NULL, ModulePath, SIZEOF(ModulePath)
  FUNCTION = LEFT$(ModulePath, INSTR(-1, ModulePath, "\")-1)
END FUNCTION

'*****************************************************************************
'                                                                            *
' CheckForPanning()                                                          *
'                                                                            *
' The CheckForPanning subroutine checks if any of the four arrow keys are    *
' pressed, and if so, pans in that direction if possible. It is called from  *
' the message loop in WinMain() when no messages are waiting.                *
'                                                                            *
'*****************************************************************************

SUB CheckForPanning()

  %kbEscape = 1
  %kbLeft = 75
  %kbRight = 77
  %kbUp = 72
  %kbDown = 80

  IF fg_kbtest(%kbLeft) AND CanGoLeft THEN
    IF X = xLimit THEN CanGoRight = %TRUE
    DECR X
    fg_vbpaste X, X+(%vbWidth-1), Y, Y+(%vbHeight-1), 0, %vbHeight-1
    IF X = 0 THEN CanGoLeft = %FALSE
  ELSEIF fg_kbtest(%kbRight) AND CanGoRight THEN
    IF X = 0 THEN CanGoLeft = %TRUE
    INCR X
    fg_vbpaste X, X+(%vbWidth-1), Y, Y+(%vbHeight-1), 0, %vbHeight-1
    IF X = xLimit THEN CanGoRight = %FALSE
  ELSEIF fg_kbtest(%kbUp) AND CanGoUp THEN
    IF Y = yLimit THEN CanGoDown = %TRUE
    DECR Y
    fg_vbpaste X, X+(%vbWidth-1), Y, Y+(%vbHeight-1), 0, %vbHeight-1
    IF Y = 0 THEN CanGoUp = %FALSE
  ELSEIF fg_kbtest(%kbDown) AND CanGoDown THEN
    IF Y = 0 THEN CanGoUp = %TRUE
    INCR Y
    fg_vbpaste X, X+(%vbWidth-1), Y, Y+(%vbHeight-1), 0, %vbHeight-1
    IF Y = yLimit THEN CanGoDown = %FALSE
  ELSEIF fg_kbtest(%kbEscape) THEN
    X = 0
    Y = 0
    fg_vbpaste(0, %vbWidth-1, 0, %vbHeight-1, 0, %vbHeight-1
    IF xLimit > 0 THEN CanGoRight = %TRUE
    IF yLimit > 0 THEN CanGoDown = %TRUE
    CanGoLeft = %FALSE
    CanGoUp = %FALSE
  END IF

END SUB
