'*****************************************************************************
'                                                                            *
'  Panner.bas                                                                *
'                                                                            *
'  This program shows how to pan the contents of a virtual buffer through    *
'  a smaller window.                                                         *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "Panner.exe"

#INCLUDE "FGWin.inc"
#RESOURCE "Panner.pbr"

%vbWidth = 640
%vbHeight = 480

DECLARE FUNCTION AppPath() AS STRING

'------------------------------------------------------------------------------
' Menu identifiers much match those in Panner.h
%IDM_Left  = 1
%IDM_Right = 2
%IDM_Up    = 3
%IDM_Down  = 4
%IDM_Reset = 5
%IDM_Exit  = 6

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  szClassName            = "FGpanner"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = VARPTR(szClassName)
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Panning Demo", _            ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %vbWidth/2, _                ' initial x size
    %vbHeight/2, _               ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hMenu AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG

GLOBAL X AS LONG, Y AS LONG
GLOBAL xLimit AS LONG, yLimit AS LONG


FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      hVB = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB
      fg_vbcolors

      fg_showbmp AppPath()+"\PORCH.BMP", 0
      x = 0
      y = 0

      hMenu = GetMenu(hWnd)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_COMMAND
      SELECT CASE wParam

        CASE %IDM_Left
          CALL Left_Click

        CASE %IDM_Right
          CALL Right_Click

        CASE %IDM_Up
          CALL Up_Click

        CASE %IDM_Down
          CALL Down_Click

        CASE %IDM_Reset
          CALL Reset_Click

        CASE %IDM_Exit
          DestroyWindow hWnd
      END SELECT
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbpaste X, X+(%vbWidth-1), Y, Y+(%vbHeight-1), 0, %vbHeight-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      IF cxClient < %vbWidth THEN
        xLimit = %vbWidth - cxClient
        IF X > 0 THEN EnableMenuItem hMenu, %IDM_Left, %MF_ENABLED
        IF X < xLimit THEN EnableMenuItem hMenu, %IDM_Right, %MF_ENABLED
      ELSE
        xLimit = 0
        EnableMenuItem hMenu, %IDM_Left, %MF_GRAYED
        EnableMenuItem hMenu, %IDM_Right, %MF_GRAYED
      END IF
      IF cyClient < %vbHeight THEN
        yLimit = %vbHeight - cyClient
        IF Y > 0 THEN EnableMenuItem hMenu, %IDM_Up, %MF_ENABLED
        IF Y < yLimit THEN EnableMenuItem hMenu, %IDM_Down, %MF_ENABLED
      ELSE
        yLimit = 0
        EnableMenuItem hMenu, %IDM_Up, %MF_GRAYED
        EnableMenuItem hMenu, %IDM_Down, %MF_GRAYED
      END IF
      DrawMenuBar hWnd
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

SUB Left_Click()
  IF X = xLimit THEN
    EnableMenuItem hMenu, %IDM_Right, %MF_ENABLED
    DrawMenuBar GetActiveWindow()
  END IF
  DECR X
  fg_vbpaste X, X+(%vbWidth-1), Y, Y+(%vbHeight-1), 0, %vbHeight-1
  IF X = 0 THEN
    EnableMenuItem hMenu, %IDM_Left, %MF_GRAYED
    DrawMenuBar GetActiveWindow()
  END IF
END SUB

SUB Right_Click()
  IF X = 0 THEN
    EnableMenuItem hMenu, %IDM_Left, %MF_ENABLED
    DrawMenuBar GetActiveWindow()
  END IF
  INCR X
  fg_vbpaste X, X+(%vbWidth-1), Y, Y+(%vbHeight-1), 0, %vbHeight-1
  IF X = xLimit THEN
    EnableMenuItem hMenu, %IDM_Right, %MF_GRAYED
    DrawMenuBar GetActiveWindow()
  END IF
END SUB

SUB Up_Click()
  IF Y = yLimit THEN
    EnableMenuItem hMenu, %IDM_Down, %MF_ENABLED
    DrawMenuBar GetActiveWindow()
  END IF
  DECR Y
  fg_vbpaste X, X+(%vbWidth-1), Y, Y+(%vbHeight-1), 0, %vbHeight-1
  IF Y = 0 THEN
    EnableMenuItem hMenu, %IDM_Up, %MF_GRAYED
    DrawMenuBar GetActiveWindow()
  END IF
END SUB

SUB Down_Click()
  IF Y = 0 THEN
    EnableMenuItem hMenu, %IDM_Up, %MF_ENABLED
    DrawMenuBar GetActiveWindow()
  END IF
  INCR Y
  fg_vbpaste X, X+(%vbWidth-1), Y, Y+(%vbHeight-1), 0, %vbHeight-1
  IF Y = yLimit THEN
    EnableMenuItem hMenu, %IDM_Down, %MF_GRAYED
    DrawMenuBar GetActiveWindow()
  END IF
END SUB

SUB Reset_Click()
  X = 0
  Y = 0
  fg_vbpaste 0, %vbWidth-1, 0, %vbHeight-1, 0, %vbHeight-1
  IF xLimit > 0 THEN EnableMenuItem hMenu, %IDM_Right, %MF_ENABLED
  IF yLimit > 0 THEN EnableMenuItem hMenu, %IDM_Down, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Left, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Up, %MF_GRAYED
  DrawMenuBar GetActiveWindow()
END SUB

FUNCTION AppPath() AS STRING
  LOCAL ModulePath AS ASCIIZ * 256
  GetModuleFileName %NULL, ModulePath, SIZEOF(ModulePath)
  FUNCTION = LEFT$(ModulePath, INSTR(-1, ModulePath, "\")-1)
END FUNCTION
