/****************************************************************************\
*                                                                            *
*  Panner.c                                                                  *
*                                                                            *
*  This program shows how to pan the contents of a virtual buffer through    *
*  a smaller window.                                                         *
*                                                                            *
\****************************************************************************/

#include <fgwin.h>
#include "Panner.h"

#define vbWidth  640
#define vbHeight 480

LRESULT CALLBACK WindowProc(HWND,UINT,WPARAM,LPARAM);

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdParam, int iCmdShow)
{
   static char szAppName[] = "FGpanner";
   HWND        hWnd;
   MSG         msg;
   WNDCLASSEX  wndclass;

   wndclass.cbSize        = sizeof(wndclass);
   wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
   wndclass.lpfnWndProc   = WindowProc;
   wndclass.cbClsExtra    = 0;
   wndclass.cbWndExtra    = 0;
   wndclass.hInstance     = hInstance;
   wndclass.hIcon         = LoadIcon(NULL,IDI_APPLICATION);
   wndclass.hCursor       = LoadCursor (NULL,IDC_ARROW);
   wndclass.hbrBackground = NULL;
   wndclass.lpszMenuName  = szAppName;
   wndclass.lpszClassName = szAppName;
   wndclass.hIconSm       = LoadIcon(NULL,IDI_APPLICATION);
   RegisterClassEx(&wndclass);

   hWnd = CreateWindow(szAppName, // window class name
      "Panning Demo",          // window caption
      WS_OVERLAPPEDWINDOW,     // window style
      CW_USEDEFAULT,           // initial x position
      CW_USEDEFAULT,           // initial y position
      vbWidth/2,               // initial x size
      vbHeight/2,              // initial y size
      NULL,                    // parent window handle
      NULL,                    // window menu handle
      hInstance,               // program instance handle
      NULL);                   // creation parameters

   ShowWindow(hWnd,iCmdShow);
   UpdateWindow(hWnd);

   while (GetMessage(&msg,NULL,0,0))
   {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }
   return msg.wParam;
}

/****************************************************************************\
*                                                                            *
*  WindowProc()                                                              *
*                                                                            *
\****************************************************************************/

HDC      hDC;
HMENU    hMenu;
HPALETTE hPal;
int      hVB;
UINT     cxClient, cyClient;
int      x, y;
int      xLimit, yLimit;

LRESULT CALLBACK WindowProc(HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
   PAINTSTRUCT ps;

   switch (iMsg)
   {
      case WM_CREATE:
         hDC = GetDC(hWnd);
         fg_setdc(hDC);
         hPal = fg_defpal();
         fg_realize(hPal);

         fg_vbinit();
         hVB = fg_vballoc(vbWidth,vbHeight);
         fg_vbopen(hVB);
         fg_vbcolors();

         fg_showbmp("PORCH.BMP",0);
         x = y = 0;

         hMenu = GetMenu(hWnd);
         return 0;

      case WM_COMMAND:
         switch (wParam)
         {
            case IDM_Left:
               if (x == xLimit)
               {
                  EnableMenuItem(hMenu,IDM_Right,MF_ENABLED);
                  DrawMenuBar(hWnd);
               }
               x--;
               fg_vbpaste(x,x+(vbWidth-1),y,y+(vbHeight-1),0,vbHeight-1);
               if (x == 0)
               {
                  EnableMenuItem(hMenu,IDM_Left,MF_GRAYED);
                  DrawMenuBar(hWnd);
               }
               return 0;

            case IDM_Right:
               if (x == 0)
               {
                  EnableMenuItem(hMenu,IDM_Left,MF_ENABLED);
                  DrawMenuBar(hWnd);
               }
               x++;
               fg_vbpaste(x,x+(vbWidth-1),y,y+(vbHeight-1),0,vbHeight-1);
               if (x == xLimit)
               {
                  EnableMenuItem(hMenu,IDM_Right,MF_GRAYED);
                  DrawMenuBar(hWnd);
               }
               return 0;

            case IDM_Up:
               if (y == yLimit)
               {
                  EnableMenuItem(hMenu,IDM_Down,MF_ENABLED);
                  DrawMenuBar(hWnd);
               }
               y--;
               fg_vbpaste(x,x+(vbWidth-1),y,y+(vbHeight-1),0,vbHeight-1);
               if (y == 0)
               {
                  EnableMenuItem(hMenu,IDM_Up,MF_GRAYED);
                  DrawMenuBar(hWnd);
               }
               return 0;

            case IDM_Down:
               if (y == 0)
               {
                  EnableMenuItem(hMenu,IDM_Up,MF_ENABLED);
                  DrawMenuBar(hWnd);
               }
               y++;
               fg_vbpaste(x,x+(vbWidth-1),y,y+(vbHeight-1),0,vbHeight-1);
               if (y == yLimit)
               {
                  EnableMenuItem(hMenu,IDM_Down,MF_GRAYED);
                  DrawMenuBar(hWnd);
               }
               return 0;

            case IDM_Reset:
               x = y = 0;
               fg_vbpaste(0,vbWidth-1,0,vbHeight-1,0,vbHeight-1);
               if (xLimit > 0) EnableMenuItem(hMenu,IDM_Right,MF_ENABLED);
               if (yLimit > 0) EnableMenuItem(hMenu,IDM_Down,MF_ENABLED);
               EnableMenuItem(hMenu,IDM_Left,MF_GRAYED);
               EnableMenuItem(hMenu,IDM_Up,MF_GRAYED);
               DrawMenuBar(hWnd);
               return 0;

            case IDM_Exit:
               DestroyWindow(hWnd);
               return 0;
         }
         break;

      case WM_PAINT:
         BeginPaint(hWnd,&ps);
         fg_vbpaste(x,x+(vbWidth-1),y,y+(vbHeight-1),0,vbHeight-1);
         EndPaint(hWnd,&ps);
         return 0;

      case WM_SETFOCUS:
         fg_realize(hPal);
         InvalidateRect(hWnd,NULL,TRUE);
         return 0;

      case WM_SIZE:
         cxClient = LOWORD(lParam);
         cyClient = HIWORD(lParam);
         if (cxClient < vbWidth)
         {
            xLimit = vbWidth - cxClient;
            if (x > 0)      EnableMenuItem(hMenu,IDM_Left,MF_ENABLED);
            if (x < xLimit) EnableMenuItem(hMenu,IDM_Right,MF_ENABLED);
         }
         else
         {
            xLimit = 0;
            EnableMenuItem(hMenu,IDM_Left,MF_GRAYED);
            EnableMenuItem(hMenu,IDM_Right,MF_GRAYED);
         }
         if (cyClient < vbHeight)
         {
            yLimit = vbHeight - cyClient;
            if (y > 0)      EnableMenuItem(hMenu,IDM_Up,MF_ENABLED);
            if (y < yLimit) EnableMenuItem(hMenu,IDM_Down,MF_ENABLED);
         }
         else
         {
            yLimit = 0;
            EnableMenuItem(hMenu,IDM_Up,MF_GRAYED);
            EnableMenuItem(hMenu,IDM_Down,MF_GRAYED);
         }
         DrawMenuBar(hWnd);
         return 0;

      case WM_DESTROY:
         fg_vbclose();
         fg_vbfree(hVB);
         fg_vbfin();
         DeleteObject(hPal);
         ReleaseDC(hWnd,hDC);
         PostQuitMessage(0);
         return 0;
   }
   return DefWindowProc(hWnd,iMsg,wParam,lParam);
}
