/****************************************************************************\
*                                                                            *
*  Panner.cs                                                                 *
*                                                                            *
*  This program shows how to pan the contents of a virtual buffer through    *
*  a smaller window.                                                         *
*                                                                            *
\****************************************************************************/
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.InteropServices;

namespace Panner
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		// Fastgraph variables.
		Graphics g;
		IntPtr hDC;
		int hPal;
		int hVB;
		int cxClient, cyClient;

		// Virtual buffer dimensions.
		const int vbWidth = 640;
		const int vbHeight = 480;

		// Application variables.
		int x, y;
		int xLimit, yLimit;

		// Component declarations.
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem menuLeft;
		private System.Windows.Forms.MenuItem menuRight;
		private System.Windows.Forms.MenuItem menuUp;
		private System.Windows.Forms.MenuItem menuDown;
		private System.Windows.Forms.MenuItem menuReset;
		private System.Windows.Forms.MenuItem menuExit;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// Add any constructor code after InitializeComponent call
			//
			g = CreateGraphics();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			g.Dispose();
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.menuLeft = new System.Windows.Forms.MenuItem();
			this.menuRight = new System.Windows.Forms.MenuItem();
			this.menuUp = new System.Windows.Forms.MenuItem();
			this.menuDown = new System.Windows.Forms.MenuItem();
			this.menuReset = new System.Windows.Forms.MenuItem();
			this.menuExit = new System.Windows.Forms.MenuItem();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuLeft,
																					  this.menuRight,
																					  this.menuUp,
																					  this.menuDown,
																					  this.menuReset,
																					  this.menuExit});
			// 
			// menuLeft
			// 
			this.menuLeft.Enabled = false;
			this.menuLeft.Index = 0;
			this.menuLeft.Text = "&Left";
			this.menuLeft.Click += new System.EventHandler(this.menuLeft_Click);
			// 
			// menuRight
			// 
			this.menuRight.Index = 1;
			this.menuRight.Text = "&Right";
			this.menuRight.Click += new System.EventHandler(this.menuRight_Click);
			// 
			// menuUp
			// 
			this.menuUp.Enabled = false;
			this.menuUp.Index = 2;
			this.menuUp.Text = "&Up";
			this.menuUp.Click += new System.EventHandler(this.menuUp_Click);
			// 
			// menuDown
			// 
			this.menuDown.Index = 3;
			this.menuDown.Text = "&Down";
			this.menuDown.Click += new System.EventHandler(this.menuDown_Click);
			// 
			// menuReset
			// 
			this.menuReset.Index = 4;
			this.menuReset.Text = "R&eset";
			this.menuReset.Click += new System.EventHandler(this.menuReset_Click);
			// 
			// menuExit
			// 
			this.menuExit.Index = 5;
			this.menuExit.Text = "E&xit";
			this.menuExit.Click += new System.EventHandler(this.menuExit_Click);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(446, 276);
			this.Menu = this.mainMenu1;
			this.Name = "Form1";
			this.Text = "Panning Demo";
			this.Resize += new System.EventHandler(this.Form1_Resize);
			this.Load += new System.EventHandler(this.Form1_Load);
			this.Closed += new System.EventHandler(this.Form1_Closed);
			this.Activated += new System.EventHandler(this.Form1_Activated);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.Form1_Paint);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		private void Form1_Activated(object sender, System.EventArgs e)
		{
			fg.realize(hPal);
			cxClient = ClientRectangle.Width;
			cyClient = ClientRectangle.Height;
			Refresh();
		}

		private void Form1_Load(object sender, System.EventArgs e)
		{
	        hDC = g.GetHdc();
			fg.setdc(hDC);
			hPal = fg.defpal();
			fg.realize(hPal);

			fg.vbinit();
			hVB = fg.vballoc(vbWidth, vbHeight);
			fg.vbopen(hVB);
			fg.vbcolors();

			fg.showbmp("..\\PORCH.BMP", 0);
			x = 0;
			y = 0;
		}

		private void Form1_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			fg.vbpaste(x, x+(vbWidth-1), y, y+(vbHeight-1), 0, vbHeight-1);
		}

		private void Form1_Resize(object sender, System.EventArgs e)
		{
			cxClient = ClientRectangle.Width;
			cyClient = ClientRectangle.Height;
			if (cxClient < vbWidth)
			{
				xLimit = vbWidth - cxClient;
				if (x > 0) menuLeft.Enabled = true;
				if (x < xLimit) menuRight.Enabled = true;
			}
			else
			{
				xLimit = 0;
				menuLeft.Enabled = false;
				menuRight.Enabled = false;
			}
			if (cyClient < vbHeight)
			{
				yLimit = vbHeight - cyClient;
				if (y > 0) menuUp.Enabled = true;
				if (y < yLimit) menuDown.Enabled = true;
			}
			else
			{
				yLimit = 0;
				menuUp.Enabled = false;
				menuDown.Enabled = false;
			}
			fg.vbpaste(x, x+(vbWidth-1), y, y+(vbHeight-1), 0, vbHeight-1);
		}

		private void Form1_Closed(object sender, System.EventArgs e)
		{
			fg.vbclose();
			fg.vbfree(hVB);
			fg.vbfin();
			g.ReleaseHdc(hDC);
		}

		private void menuLeft_Click(object sender, System.EventArgs e)
		{
			if (x == xLimit) menuRight.Enabled = true;
			x--;
			fg.vbpaste(x, x+(vbWidth-1), y, y+(vbHeight-1), 0, vbHeight-1);
			if (x == 0) menuLeft.Enabled = false;
		}

		private void menuRight_Click(object sender, System.EventArgs e)
		{
			if (x == 0) menuLeft.Enabled = true;
			x++;
			fg.vbpaste(x, x+(vbWidth-1), y, y+(vbHeight-1), 0, vbHeight-1);
			if (x == xLimit) menuRight.Enabled = false;
		}

		private void menuUp_Click(object sender, System.EventArgs e)
		{
			if (y == yLimit) menuDown.Enabled = true;
			y--;
			fg.vbpaste(x, x+(vbWidth-1), y, y+(vbHeight-1), 0, vbHeight-1);
			if (y == 0) menuUp.Enabled = false;
		}

		private void menuDown_Click(object sender, System.EventArgs e)
		{
			if (y == 0) menuUp.Enabled = true;
			y++;
			fg.vbpaste(x, x+(vbWidth-1), y, y+(vbHeight-1), 0, vbHeight-1);
			if (y == yLimit) menuDown.Enabled = false;
		}

		private void menuReset_Click(object sender, System.EventArgs e)
		{
			x = 0;
			y = 0;
			fg.vbpaste(0, vbWidth-1, 0, vbHeight-1, 0, vbHeight-1);
			if (xLimit > 0) menuRight.Enabled = true;
			if (yLimit > 0) menuDown.Enabled = true;
			menuLeft.Enabled = false;
			menuUp.Enabled = false;
		}

		private void menuExit_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}
	}
}