'*****************************************************************************
'                                                                            *
'  Rainbow.bas                                                               *
'                                                                            *
'  This program demonstrates color palette cycling.                          *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "Rainbow.exe"

#INCLUDE "FGWin.inc"

GLOBAL RGBvalues() AS BYTE

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  DIM RGBvalues(2*24*3)         ' two sets of 24 RGB triplets

  szClassName            = "FGrainbow"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = %NULL
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Color Cycling", _           ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG
GLOBAL Start AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT
  LOCAL nColor AS LONG, xLen AS LONG, yLen AS LONG

  SELECT CASE iMsg

    CASE %WM_CREATE
      ' create the logical palette
      hDC = GetDC(hWnd)
      fg_setdc hDC
      CALL FillColorPalette
      hPal = fg_logpal(10, 24, RGBvalues(0))
      fg_realize hPal

      ' create a 640x480 virtual buffer
      fg_vbinit
      hVB = fg_vballoc(640, 480)
      fg_vbopen hVB
      fg_vbcolors

      ' construct a crude image of a rainbow
      fg_setcolor 255
      fg_fillpage
      fg_setclip 0, 639, 0, 300
      fg_move 320, 300
      xLen = 240
      yLen = 120
      FOR nColor = 10 TO 33
        fg_setcolor nColor
        fg_ellipsef xLen, yLen
        xLen = xLen - 4
        yLen = yLen - 3
      NEXT nColor
      fg_setcolor 255
      fg_ellipsef xLen, yLen
      fg_setclip 0, 639, 0, 479

      ' starting index into the array of color values
      Start = 0

      ' start the 50ms timer
      SetTimer hWnd, 1, 50, %NULL
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, fg_getmaxx(), 0, fg_getmaxy(), 0, cxClient-1, 0, cyClient-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_TIMER
      IF GetActiveWindow() = hWnd THEN
        Start = (Start + 3) MOD 72
        fg_setdacs 10, 24, RGBvalues(Start)
        IF fg_colors() > 8 THEN
          fg_vbscale 0, fg_getmaxx(), 0, fg_getmaxy(), 0, cxClient-1, 0, cyClient-1
        END IF
      END IF
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      KillTimer hWnd, 1
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION


'*****************************************************************************
'                                                                            *
'  FillColorPalette()                                                        *
'                                                                            *
'  Set up the colors for the application's logical palette in the RGBvalues  *
'  array. The logical palette will contain 24 non-system colors (indices 10  *
'  to 33) defining the initial RGB values for the colors being cycled.       *
'                                                                            *
'  Note that we store two identical sets of 24 RGB triplets in RGBvalues. We *
'  can then perform color cycling without having to worry about wrapping to  *
'  the start of the array because the index pointing to the starting RGB     *
'  triplet never extends beyond the first set of 24 RGB triplets.            *
'                                                                            *
'*****************************************************************************

SUB FillColorPalette()

   DIM I AS LONG

   RGBvalues(0)  = 182: RGBvalues(1)  = 182: RGBvalues(2)  = 255
   RGBvalues(3)  = 198: RGBvalues(4)  = 182: RGBvalues(5)  = 255
   RGBvalues(6)  = 218: RGBvalues(7)  = 182: RGBvalues(8)  = 255
   RGBvalues(9)  = 234: RGBvalues(10) = 182: RGBvalues(11) = 255
   RGBvalues(12) = 255: RGBvalues(13) = 182: RGBvalues(14) = 255
   RGBvalues(15) = 255: RGBvalues(16) = 182: RGBvalues(17) = 234
   RGBvalues(18) = 255: RGBvalues(19) = 182: RGBvalues(20) = 218
   RGBvalues(21) = 255: RGBvalues(22) = 182: RGBvalues(23) = 198
   RGBvalues(24) = 255: RGBvalues(25) = 182: RGBvalues(26) = 182
   RGBvalues(27) = 255: RGBvalues(28) = 198: RGBvalues(29) = 182
   RGBvalues(30) = 255: RGBvalues(31) = 218: RGBvalues(32) = 182
   RGBvalues(33) = 255: RGBvalues(34) = 234: RGBvalues(35) = 182
   RGBvalues(36) = 255: RGBvalues(37) = 255: RGBvalues(38) = 182
   RGBvalues(39) = 234: RGBvalues(40) = 255: RGBvalues(41) = 182
   RGBvalues(42) = 218: RGBvalues(43) = 255: RGBvalues(44) = 182
   RGBvalues(45) = 198: RGBvalues(46) = 255: RGBvalues(47) = 182
   RGBvalues(48) = 182: RGBvalues(49) = 255: RGBvalues(50) = 182
   RGBvalues(51) = 182: RGBvalues(52) = 255: RGBvalues(53) = 198
   RGBvalues(54) = 182: RGBvalues(55) = 255: RGBvalues(56) = 218
   RGBvalues(57) = 182: RGBvalues(58) = 255: RGBvalues(59) = 234
   RGBvalues(60) = 182: RGBvalues(61) = 255: RGBvalues(62) = 255
   RGBvalues(63) = 182: RGBvalues(64) = 234: RGBvalues(65) = 255
   RGBvalues(66) = 182: RGBvalues(67) = 218: RGBvalues(68) = 255
   RGBvalues(69) = 182: RGBvalues(70) = 198: RGBvalues(71) = 255

   ' set up two identical sets of the 24 colors in the RGBvalues array
   FOR I = 0 TO 24*3-1
      RGBvalues(I+24*3) = RGBvalues(I)
   NEXT I

END SUB
