// MainFrame.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "TMcube.h"

#include "MainFrame.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

typedef struct point3d
{
	double x;
	double y;
	double z;
} POINT3D;

// six faces of a 40x40x40 cube, defined in object coordinates
POINT3D Face1[] = {
	{ 20.0,-20.0,-20.0},
	{-20.0,-20.0,-20.0},
	{-20.0, 20.0,-20.0},
	{ 20.0, 20.0,-20.0}
};
POINT3D Face2[] = {
	{-20.0,-20.0,-20.0},
	{-20.0,-20.0, 20.0},
	{-20.0, 20.0, 20.0},
	{-20.0, 20.0,-20.0}
};
POINT3D Face3[] = {
	{ 20.0, 20.0, 20.0},
	{-20.0, 20.0, 20.0},
	{-20.0,-20.0, 20.0},
	{ 20.0,-20.0, 20.0}
};
POINT3D Face4[] = {
	{ 20.0,-20.0, 20.0},
	{ 20.0,-20.0,-20.0},
	{ 20.0, 20.0,-20.0},
	{ 20.0, 20.0, 20.0}
};
POINT3D Face5[] = {
	{ 20.0,-20.0, 20.0},
	{-20.0,-20.0, 20.0},
	{-20.0,-20.0,-20.0},
	{ 20.0,-20.0,-20.0}
};
POINT3D Face6[] = {
	{ 20.0, 20.0,-20.0},
	{-20.0, 20.0,-20.0},
	{-20.0, 20.0, 20.0},
	{ 20.0, 20.0, 20.0}
};

// for convenience, an array of pointers to each of the six faces
POINT3D *Faces[] = {Face1,Face2,Face3,Face4,Face5,Face6};

// texture map array
BYTE Texture[6][tmWidth*tmWidth*(vbDepth/8)];

// coordinates defining source polygon vertices within the texture map array
int tmSource[] = {tmWidth-1,tmWidth-1, 0,tmWidth-1, 0,0, tmWidth-1,0};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CFrameWnd)
BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_SIZE()
	ON_WM_SETFOCUS()
	ON_WM_QUERYNEWPALETTE()
	ON_WM_PALETTECHANGED()
	ON_WM_PAINT()
	ON_WM_ACTIVATEAPP()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_hDC = NULL;
	m_hPal = NULL;
	m_hVB = -1;
	m_cxClient = 0;
	m_cyClient = 0;
	m_hZB = -1;
	m_bAppIsActive = FALSE;
}

CMainFrame::~CMainFrame()
{
	if (m_hVB >= 0)
	{
		fg_vbclose();
		fg_vbfree(m_hVB);
	}
	fg_tmfree(-1);
	if (m_hZB >= 0)
	{
		fg_zbfree(m_hZB);
	}
	fg_vbfin();
	if (m_hPal)
	{
		DeleteObject(m_hPal);
		m_hPal = NULL;
	}
	if (m_hDC)
	{
		::ReleaseDC(m_hWnd, m_hDC);
		m_hDC = NULL;
	}
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if (!CFrameWnd::PreCreateWindow(cs))
		return FALSE;

	cs.style = WS_OVERLAPPEDWINDOW | FWS_ADDTOTITLE;
	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	cs.lpszClass = AfxRegisterWndClass(CS_OWNDC|CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS,
		LoadCursor(NULL,IDC_ARROW), NULL, NULL);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext)
{
	register int i;
	BOOL bRet = CFrameWnd::OnCreateClient(lpcs, pContext);
	if (bRet)
	{
		m_hDC = ::GetDC(m_hWnd);
		fg_setdc(m_hDC);
		m_hPal = fg_defpal();
		fg_realize(m_hPal);

		fg_vbinit();
		fg_vbdepth(vbDepth);
		m_hVB = fg_vballoc(vbWidth,vbHeight);
		fg_vbopen(m_hVB);
		fg_vbcolors();

		m_hZB = fg_zballoc(vbWidth,vbHeight);
		fg_zbopen(m_hZB);

		// define 3D viewport, clipping planes, and initial render state
		fg_3Dviewport(0,vbWidth-1,0,vbHeight-1,0.5);
		fg_3Dsetzclip(40.0,1000.0);
		fg_3Drenderstate(RENDER_STATE);

		// obtain the six texture maps from the CUBE.PCX file
		fg_tminit(6);
		fg_showpcx("CUBE.PCX",FG_AT_XY|FG_KEEPCOLORS);
		fg_move(0,tmWidth-1);
		for (i = 0; i < 6; i++)
		{
#if (vbDepth == 8)
			fg_getimage(Texture[i],tmWidth,tmWidth);
			fg_invert(Texture[i],tmWidth,tmWidth);
#else
			fg_getdcb(Texture[i],tmWidth,tmWidth);
			fg_invdcb(Texture[i],tmWidth,tmWidth);
#endif
			m_hTM[i] = fg_tmdefine(Texture[i],tmWidth,tmWidth);
			fg_moverel(tmWidth,0);
		}

		fg_setcolor(-1);
		fg_fillpage();

		m_bAppIsActive = TRUE;
	}
	return bRet;
}

void CMainFrame::OnPaint()
{
	CPaintDC dc(this); // device context for painting

	fg_vbscale(0,vbWidth-1,0,vbHeight-1,0,m_cxClient-1,0,m_cyClient-1);
}

void CMainFrame::OnSize(UINT, int cx, int cy)
{
	m_cxClient = cx;
	m_cyClient = cy;
}

void CMainFrame::OnSetFocus(CWnd* pOldWnd)
{
	OnQueryNewPalette();
}

BOOL CMainFrame::OnQueryNewPalette()
{
	fg_realize(m_hPal);
	Invalidate();
	return TRUE;
}

void CMainFrame::OnPaletteChanged(CWnd* pFocusWnd)
{
	if ((pFocusWnd != this) && (!IsChild(pFocusWnd)))
		OnQueryNewPalette();
}

void CMainFrame::OnActivateApp(BOOL bActive, HTASK hTask)
{
	CFrameWnd::OnActivateApp(bActive, hTask);
	m_bAppIsActive = bActive;
}

/****************************************************************************\
*                                                                            *
*  CheckForMovement()                                                        *
*                                                                            *
*  The CheckForMovement() function checks for key presses that control the   *
*  cube's movement, and if required redraws the cube at its new position     *
*  and orientation. It is called from CTMcubeApp::Run() when there are no    *
*  messages waiting.                                                         *
*                                                                            *
\****************************************************************************/

void CMainFrame::CheckForMovement()
{
	static double xWorld = 0.0, yWorld = 0.0, zWorld = 100.0;
	static int xAngle = 0, yAngle = 0, zAngle = 0;
	static BOOL Redraw = TRUE;
	BOOL ShiftKey;

	// check if either shift key is pressed
	ShiftKey = fg_kbtest(42) | fg_kbtest(54);

	// + and - move cube along the z axis (+ is toward viewer, - is
	// away from viewer)
	if (fg_kbtest(74))
	{
		zWorld += 3.0;
		Redraw = TRUE;
	}
	else if (fg_kbtest(78))
	{
		zWorld -= 3.0;
		Redraw = TRUE;
	}

	// left and right arrow keys move cube along x axis
	else if (fg_kbtest(75))
	{
		xWorld -= 3.0;
		Redraw = TRUE;
	}
	else if (fg_kbtest(77))
	{
		xWorld += 3.0;
		Redraw = TRUE;
	}

	// up and down arrow keys move cube along y axis
	else if (fg_kbtest(72))
	{
		yWorld += 3.0;
		Redraw = TRUE;
	}
	else if (fg_kbtest(80))
	{
		yWorld -= 3.0;
		Redraw = TRUE;
	}

	// x rotates counterclockwise around x axis, X rotates clockwise
	else if (fg_kbtest(45))
	{
		if (ShiftKey)
		{
			xAngle += 6;
			if (xAngle >= 360) xAngle -= 360;
		}
		else
		{
			xAngle -= 6;
			if (xAngle < 0) xAngle += 360;
		}
		Redraw = TRUE;
	}

	// y rotates counterclockwise around y axis, Y rotates clockwise
	else if (fg_kbtest(21))
	{
		if (ShiftKey)
		{
			yAngle += 6;
			if (yAngle >= 360) yAngle -= 360;
		}
		else
		{
			yAngle -= 6;
			if (yAngle < 0) yAngle += 360;
		}
		Redraw = TRUE;
	}

	// z rotates counterclockwise around z axis, Z rotates clockwise
	else if (fg_kbtest(44))
	{
		if (ShiftKey)
		{
			zAngle += 6;
			if (zAngle >= 360) zAngle -= 360;
		}
		else
		{
			zAngle -= 6;
			if (zAngle < 0) zAngle += 360;
		}
		Redraw = TRUE;
	}

	// if the cube's position or rotation changed, redraw the cube
	if (Redraw)
	{
		// prepare the z-buffer for the next frame
		fg_zbframe();

		// erase the previous frame from the virtual buffer
		fg_setcolor(-1);
		fg_fillpage();

		// define the cube's new position and rotation in 3D world space
		fg_3Dsetobject(xWorld,yWorld,zWorld,xAngle*10,yAngle*10,zAngle*10);

		// draw the cube
		DrawCube();

		// display what we just drew
		fg_vbscale(0,vbWidth-1,0,vbHeight-1,0,m_cxClient-1,0,m_cyClient-1);
		Redraw = FALSE;
	}
}

/****************************************************************************\
*                                                                            *
*  DrawCube()                                                                *
*                                                                            *
*  Draws each of the six cube faces in 3D world space.                       *
*                                                                            *
\****************************************************************************/

void CMainFrame::DrawCube()
{
	register int i;

	for (i = 0; i < 6; i++)
	{
		fg_tmselect(m_hTM[i]);
		fg_3Dtexturemapobject((double *)Faces[i],tmSource,4);
	}
}
