'*****************************************************************************
'                                                                            *
'  VBdemo.bas                                                                *
'                                                                            *
'  This program demonstrates how to copy the contents of one virtual buffer  *
'  to another, with and without transparent colors.                          *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "VBdemo.exe"

#INCLUDE "FGWin.inc"
#RESOURCE "VBdemo.pbr"

%vbWidth = 640
%vbHeight = 480

'------------------------------------------------------------------------------
' Menu identifiers much match those in VBdemo.h
%IDM_Cut   = 1
%IDM_Paste = 2
%IDM_Exit  = 3

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL winclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  szClassName            = "FGvbdemo"

  winclass.cbSize        = SIZEOF(winclass)
  winclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  winclass.lpfnWndProc   = CODEPTR(WindowProc)
  winclass.cbClsExtra    = 0
  winclass.cbWndExtra    = 0
  winclass.hInstance     = hInstance
  winclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  winclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  winclass.hbrBackground = %NULL
  winclass.lpszMenuName  = VARPTR(szClassName)
  winclass.lpszClassName = VARPTR(szClassName)
  winclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx winclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Virtual Buffer Copy Demo", _  ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hMenu AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB1 AS LONG, hVB2 AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      hVB2 = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB2
      fg_vbcolors
      hVB1 = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB1
      fg_vbcolors

      fg_setcolor 25
      fg_fillpage
      fg_setcolor 20
      fg_rect %vbWidth/4, %vbWidth*3/4, %vbHeight/4, %vbHeight*3/4

      hMenu = GetMenu(hWnd)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_COMMAND
      SELECT CASE wParam

        CASE %IDM_Cut
          CALL Cut_Click
          EnableMenuItem hMenu, %IDM_Paste, %MF_ENABLED
          DrawMenuBar hWnd

        CASE %IDM_Paste
          CALL Paste_Click

        CASE %IDM_Exit
          DestroyWindow hWnd
      END SELECT
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB1
      fg_vbfree hVB2
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

SUB Cut_Click()
  fg_vbcopy 0, %vbWidth-1, 0, %vbHeight-1, 0, %vbHeight-1, hVB1, hVB2
  fg_erase
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

SUB Paste_Click()
  fg_tcdefine 25, 1
  fg_vbtccopy 0, %vbWidth-1, 0, %vbHeight-1, 0, %vbHeight-1, hVB2, hVB1
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB
